<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: TestOfDalStatistics.php 2024 2005-03-10 17:21:43Z andrew $
*/

/**
 * @package    MaxDal
 * @author     Demian Turner <demian@awarez.net>
 */

require_once MAX_PATH . '/lib/max/DB.php';
require_once MAX_PATH . '/lib/max/other/common.php';

class Admin_DA
{
    // +---------------------------------------+
    // | entity builder methods                |
    // |                                       |
    // | private methods for entity            |
    // | manipulation                          |
    // +---------------------------------------+

    /**
     * Adds an entity.
     *
     * @param string $entity
     * @param array $aVariables
     * @return integer  The inserted entity's ID
     */
    function _addEntity($entity, $aVariables)
    {
        $aTable = Admin_DA::_getPrimaryTable($entity);
        return Admin_DA::_insert($aTable, $aVariables);
    }
    
    /**
     * Retrieves an entity.
     *
     * @param string $entity
     * @param integer $id
     * @return array    An array representing the attributes of an entity
     */
    function _getEntity($entity, $id)
    {
        $entityIdName = "{$entity}_id";
        $aParams = array($entityIdName => $id);
        $aRows = Admin_DA::_getEntities($entity, $aParams, true, $entityIdName);
    
        if (!empty($aRows[$id])) {
            $aRow = $aRows[$id];
        } elseif (!($aRows === false)) {
            $aRow = false;
        } else {
			$aRow = array();
        }
        return $aRow;
    }
    
    /**
     * Returns an array of entity arrays where the keys are the entity IDs.
     *
     * @param string $entity
     * @param array $aParams
     * @param boolean $allFields
     * @param string $key   The primary key field name
     * @return array    An array of entity records
     */
    function _getEntities($entity, $aParams, $allFields = false, $key = null)
    {
        if (empty($key)) {
            $key = "{$entity}_id";
        }
        $aColumns = Admin_DA::_getColumns($entity, $allFields);
        $aTables = Admin_DA::_getTables($entity, $aParams);
        $aLimitations = array_merge(
            Admin_DA::_getLimitations($entity, $aParams), 
            Admin_DA::_getTableLimitations($aTables));
        return Admin_DA::_select($aColumns, $aTables, $aLimitations, null, $key);
    }
    
    /**
     * Returns an array of the entity's children.
     *
     * @param string $entity
     * @param array $aParams
     * @param boolean $allFields
     * @param string $key   The primary key field name
     * @return array    An array of entity records
     */
    function _getEntitiesChildren($entity, $aParams, $allFields=false, $key=null)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
    
        if (empty($key)) {
            $key = "{$entity}_id";
        }
        $aColumns = Admin_DA::_getColumns($entity, $allFields);
        $aTables = Admin_DA::_getTables($entity, $aParams);
        switch ($entity) {
        
        case 'agency' : $aTables[$conf['table']['prefix'].$conf['table']['clients']] = 'a';
            $aGroupBy = $aColumns;
            $aColumns['COUNT(a.clientid)'] = 'num_children';
            break;
            
        case 'advertiser' : $aTables[$conf['table']['prefix'].$conf['table']['campaigns']] = 'm';
            $aGroupBy = $aColumns;
            $aColumns['COUNT(m.campaignid)'] = 'num_children';
            break;
            
        case 'placement' : $aTables[$conf['table']['prefix'].$conf['table']['banners']] = 'd';
            $aGroupBy = $aColumns;
            $aColumns['COUNT(d.bannerid)'] = 'num_children';
            break;
            
        case 'publisher' : $aTables[$conf['table']['prefix'].$conf['table']['zones']] = 'z';
            $aGroupBy = $aColumns;
            $aColumns['COUNT(z.zoneid)'] = 'num_children';
            break;
            
        default :
            $aGroupBy = null;
        }
        $aLimitations = array_merge(
            Admin_DA::_getLimitations($entity, $aParams), 
            Admin_DA::_getTableLimitations($aTables));
        return Admin_DA::_select($aColumns, $aTables, $aLimitations, $aGroupBy, $key);
    }    
    
    /**
     * Modifies an entity's properties.
     *
     * @param string $entity
     * @param integer $id
     * @param array $aVariables
     * @return integer  The number of rows affected by the update
     */
    function _setEntity($entity, $id, $aVariables)
    {
        $aParams = array("{$entity}_id" => $id);
        return Admin_DA::_setEntities($entity, $aParams, $aVariables);
    }
    
    
    /**
     * Modifies multiple entities.
     *
     * @param string $entity
     * @param array $aParams
     * @param array $aVariables
     */
    function _setEntities($entity, $aParams, $aVariables)
    {
        $aTable = Admin_DA::_getPrimaryTable($entity);
        $aLimitations = Admin_DA::_getLimitations($entity, $aParams);
        Admin_DA::_update($aTable, $aVariables, $aLimitations);
    }

    //  MAX_removeEntity
    /**
     * Remove the entity from the system.
     *
     * @param string $entity
     * @param integer $id
     * @return integer  The number of rows affected by the update
     */
    function _deleteEntity($entity, $id)
    {
        $aParams = array("{$entity}_id" => $id);
        return Admin_DA::_deleteEntities($entity, $aParams);
    }
    
    //  MAX_removeEntities
    /**
     * Remove a list of entities from the system.
     *
     * @param string $entity
     * @param array $aParams
     * @return integer  The number of rows affected by the update
     */
    function _deleteEntities($entity, $aParams)
    {
        $aTable = Admin_DA::_getPrimaryTable($entity);
        $aOtherTables = Admin_DA::_getTables($entity, $aParams);
        $aLimitations = array_merge(Admin_DA::_getLimitations($entity, $aParams), 
            Admin_DA::_getTableLimitations($aOtherTables));
        return Admin_DA::_delete($aTable, $aLimitations, $aOtherTables);
    }
    
    /**
     * Returns a cached entity.
     *
     * @param string $entity
     * @param integer $id
     * @param integer $timeout  Number of seconds of cache age
     * @return array    An array representing the attributes of an entity
     */
    function _getCachedEntity($entity, $id, $timeout=-1)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        include_once 'Cache/Lite/Function.php';
        $options = array('cacheDir' => MAX_CACHE, 'lifeTime' => ($timeout > -1 ? $timeout : $phpAds_config['cacheExpire']));
        $cache = new Cache_Lite_Function($options);
        return $cache->call('Admin_DA::_getEntity', $entity, $id);
    }
    
    /**
     * Returns multiple cached entities.
     *
     * @param string $entity
     * @param array $aParams
     * @param boolean $allFields
     * @param string $key   The primary key field name
     * @param integer $timeout
     * @return array    An array of entity records
     */
    function _getCachedEntities($entity, $aParams, $allFields=false, $key=null, $timeout=-1)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
    
        include_once 'Cache/Lite/Function.php';
        $options = array('cacheDir' => MAX_CACHE, 'lifeTime' => ($timeout > -1 ? $timeout : $phpAds_config['cacheExpire']));
        $cache = new Cache_Lite_Function($options);
        return $cache->call('Admin_DA::_getEntities', $entity, $aParams, $allFields, $key);
    }    
    
    /**
     * Get stats by entity.
     *
     * @param string $entity
     * @param array $aParams
     * @param boolean $allFields
     * @param string $key   The primary key field name
     * @return array    An array of entity records
     */
    function _getEntitiesStats($entity, $aParams, $allFields=false, $key=null)
    {
        if (empty($key)) {
            $key = "{$entity}_id";
        }
        $aColumns = Admin_DA::_getStatsColumns($entity);
        $aTables = Admin_DA::_getTables($entity, $aParams, true);
        $aLimitations = array_merge(Admin_DA::_getStatsLimitations($entity, $aParams), 
            Admin_DA::_getTableLimitations($aTables));
        $aGroups = array("{$entity}_id");
        return Admin_DA::_select($aColumns, $aTables, $aLimitations, $aGroups, $key);
    }
    
    /**
     * Get cached stats by entity.
     *
     * @param string $entity
     * @param array $aParams
     * @param boolean $allFields
     * @param string $key   The primary key field name
     * @param unknown_type $timeout
     * @return array    An array of entity records
     */
    function _getCachedEntitiesStats($entity, $aParams, $allFields=false, $key=null, $timeout=-1)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
    
        include_once 'Cache/Lite/Function.php';
        $options = array('cacheDir' => MAX_CACHE, 'lifeTime' => ($timeout > -1 ? $timeout : $phpAds_config['cacheExpire']));
        $cache = new Cache_Lite_Function($options);
        return $cache->call('Admin_DA::_getEntitiesStats', $entity, $aParams, $allFields, $key);
    }    

    
    
    
    
    
    
    
    

    // +---------------------------------------+
    // | Relational/constraint methods         |
    // |                                       |
    // | for entity builders                   |
    // +---------------------------------------+

//  not used anywhere
#    function _addZeroStats(&$aRow, $id)
#    {
#        if (empty($aRow['sum_requests'])) $aRow['sum_requests'] = 0;
#        if (empty($aRow['sum_views'])) $aRow['sum_views'] = 0;
#        if (empty($aRow['sum_clicks'])) $aRow['sum_clicks'] = 0;
#        if (empty($aRow['sum_conversions'])) $aRow['sum_conversions'] = 0;
#    }
    
    /**
     * Returns the fields given the particular entity.
     *
     * @param string $entity
     * @param boolean $allFields
     * @return array
     */
    function _getColumns($entity, $allFields)
    {
        $aColumns = array();
        switch ($entity) {
        
        case 'ad' :
            $aColumns += array('d.bannerid' => 'ad_id', 'd.campaignid' => 'placement_id', 'd.active' => 'active', 'd.description' => 'name', 'd.storagetype' => 'type');
            if ($allFields) $aColumns += array('d.contenttype' => 'contenttype', 'd.pluginversion' => 'pluginversion', 'd.filename' => 'filename', 'd.imageurl' => 'imageurl', 'd.htmltemplate' => 'htmltemplate', 'd.htmlcache' => 'htmlcache', 'd.width' => 'width', 'd.height' => 'height', 'd.weight' => 'weight', 'd.seq' => 'seq', 'd.target' => 'target', 'd.url' => 'url', 'd.alt' => 'alt', 'd.status' => 'status', 'd.bannertext' => 'bannertext', 'd.autohtml' => 'autohtml', 'd.adserver' => 'adserver', 'd.block' => 'block', 'd.capping' => 'capping', 'd.session_capping' => 'session_capping', 'd.compiledlimitation' => 'compiledlimitation', 'd.append' => 'append', 'd.appendtype' => 'appendtype', 'd.bannertype' => 'bannertype', 'd.alt_filename' => 'alt_filename', 'd.alt_imageurl' => 'alt_imageurl', 'd.alt_contenttype' => 'alt_contenttype');
            break;

        case 'advertiser' :
            $aColumns += array('a.clientid' => 'advertiser_id', 'a.agencyid' => 'agency_id', 'a.clientname' => 'name');
            if ($allFields) $aColumns += array('a.contact' => 'contact', 'a.email' => 'email', 'a.clientusername' => 'username', 'a.clientpassword' => 'password', 'a.permissions' => 'permissions', 'a.language' => 'language', 'a.report' => 'report', 'a.reportinterval' => 'report_interval', 'a.reportlastdate' => 'report_last_date', 'a.reportdeactivate' => 'report_deactivate');
            break;
            
        case 'ad_category_assoc' :
            $aColumns += array('ac.ad_category_assoc_id' => 'ad_category_assoc_id', 'ac.ad_id' => 'ad_id', 'ac.category_id' => 'category_id');
            break;
            
        case 'ad_zone_assoc' :
            $aColumns += array('az.ad_zone_assoc_id' => 'ad_zone_assoc_id', 'az.ad_id' => 'ad_id', 'az.zone_id' => 'zone_id', 'az.priority' => 'priority');
            break;
            
        case 'agency' :
            $aColumns += array('g.agencyid' => 'agency_id', 'g.name' => 'name');
            if ($allFields) $aColumns += array('g.contact' => 'contact', 'g.email' => 'email', 'g.username' => 'username', 'g.password' => 'password', 'g.permissions' => 'permissions', 'g.language' => 'language');
            break;
            
        case 'category' :
            $aColumns += array('cat.category_id' => 'category_id', 'cat.name' => 'name');
            break;
            
        case 'image' :
            $aColumns += array('i.filename' => 'file_name');
            if ($allFields) $aColumns += array('i.t_stamp' => 't_stamp', 'i.contents' => 'contents');
            break;
            
        case 'limitation' :
            $aColumns += array('l.bannerid' => 'ad_id', 'l.logical' => 'logical', 'l.type' => 'type', 'l.comparison' => 'comparison', 'l.data' => 'data', 'l.executionorder' => 'executionorder');
            break;
            
        case 'placement' :
            $aColumns += array('m.clientid' => 'advertiser_id', 'm.campaignid' => 'placement_id', 'm.campaignname' => 'name', 'm.active' => 'active');
            if ($allFields) $aColumns += array('m.views' => 'views', 'm.clicks' => 'clicks', 'm.conversions' => 'conversions', 'm.expire' => 'expire', 'm.activate' => 'activate', 'm.priority' => 'priority', 'm.weight' => 'weight', 'm.target' => 'target', 'm.optimise' => 'optimise', 'm.anonymous' => 'anonymous');
            break;
            
        case 'placement_tracker' :
            $aColumns += array('mt.campaign_trackerid' => 'placement_tracker_id', 'mt.campaignid' => 'placement_id', 'mt.trackerid' => 'tracker_id');
            if ($allFields) $aColumns += array('mt.logstats' => 'log_stats', 'mt.viewwindow' => 'view_window', 'mt.clickwindow' => 'click_window');
            break;
            
        case 'publisher' :
            $aColumns += array('p.affiliateid' => 'publisher_id', 'p.agencyid' => 'agency_id', 'p.name' => 'name');
            if ($allFields) $aColumns += array('p.mnemonic' => 'mnemonic', 'p.contact' => 'contact', 'p.email' => 'email', 'p.website' => 'website', 'p.username' => 'username', 'p.password' => 'password', 'p.permissions' => 'permissions', 'p.language' => 'language', 'p.publiczones' => 'publiczones');
            break;
            
        case 'stats' :
            $aColumns += array('s.day' => 'day', 's.hour' => 'hour', 'SUM(s.requests)' => 'sum_requests', 'SUM(s.impressions)' => 'sum_views', 'SUM(s.clicks)' => 'sum_clicks', 'SUM(s.conversions)' => 'sum_conversions');
            break;
            
        case 'tracker' :
            $aColumns += array('t.clientid' => 'advertiser_id', 't.trackerid' => 'tracker_id', 't.trackername' => 'name');
            if ($allFields) $aColumns += array('t.description' => 'description', 't.viewwindow' => 'viewwindow', 't.clickwindow' => 'clickwindow', 't.blockwindow' => 'blockwindow');
            break;
            
        case 'variable' :
            $aColumns += array('v.variableid' => 'variable_id', 'v.trackerid' => 'tracker_id', 'v.name' => 'name', 'v.datatype' => 'type');
            if ($allFields) $aColumns += array('v.description' => 'description', 'v.variabletype' => 'variable_type');
            break;
            
        case 'zone' :
            $aColumns += array('z.zoneid' => 'zone_id', 'z.affiliateid' => 'publisher_id', 'z.zonename' => 'name', 'z.delivery' => 'type');
            if ($allFields) $aColumns += array('z.description' => 'description', 'z.width' => 'width', 'z.height' => 'height', 'z.chain' => 'chain', 'z.prepend' => 'prepend', 'z.append' => 'append', 'z.appendtype' => 'appendtype', 'z.forceappend' => 'forceappend');
            break;
            
        case 'placement_zone_assoc' :
            $aColumns += array('pz.placement_zone_assoc_id' => 'placement_zone_assoc_id', 'pz.placement_id' => 'placement_id', 'pz.zone_id' => 'zone_id');
            break;
        }
        return $aColumns;
    }
    
    /**
     * Returns the fields required by a stats query for a given entity.
     *
     * @param string $entity
     * @return array
     */
    function _getStatsColumns($entity)
    {
        $aColumns = array('SUM(s.requests)' => 'sum_requests', 'SUM(s.impressions)' => 'sum_views', 'SUM(s.clicks)' => 'sum_clicks', 'SUM(s.conversions)' => 'sum_conversions');
        switch ($entity) {
        
        case 'ad' :         $aColumns += array('d.bannerid' => 'ad_id'); break;
        case 'advertiser' : $aColumns += array('a.clientid' => 'advertiser_id'); break;
        case 'agency' :     $aColumns += array('g.agencyid' => 'agency_id'); break;
        case 'placement' :  $aColumns += array('m.campaignid' => 'placement_id'); break;
        case 'publisher' :  $aColumns += array('p.affiliateid' => 'publisher_id'); break;
        case 'zone' :       $aColumns += array('z.zoneid' => 'zone_id'); break;
        }
    
        return $aColumns;
    }
    
    /**
     * Returns as array of table names and their aliases.
     *
     * @param string $entity
     * @param array $aParams
     * @param boolean $includeStats
     * @return array
     */
    function _getTables($entity, $aParams, $includeStats = false)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
    
        $aTables = array();
        switch ($entity) {
        
        case 'ad' :
            $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['agency_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['clients'] => 'a', $conf['table']['prefix'].$conf['table']['campaigns'] => 'm');
            if (!empty($aParams['advertiser_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['campaigns'] => 'm');
            if ($includeStats) $aTables += array($conf['table']['prefix'].$conf['table']['data_summary_ad_hourly'] => 's');
            break;
        
        case 'advertiser' :
            $aTables += array($conf['table']['prefix'].$conf['table']['clients'] => 'a');
            if (!empty($aParams['placement_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['campaigns'] => 'm');
            if (!empty($aParams['placement_active'])) $aTables += array($conf['table']['prefix'].$conf['table']['campaigns'] => 'm');
            if (!empty($aParams['ad_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['campaigns'] => 'm', $conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['ad_active'])) $aTables += array($conf['table']['prefix'].$conf['table']['campaigns'] => 'm', $conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['ad_width'])) $aTables += array($conf['table']['prefix'].$conf['table']['campaigns'] => 'm', $conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['ad_height'])) $aTables += array($conf['table']['prefix'].$conf['table']['campaigns'] => 'm', $conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['ad_type'])) $aTables += array($conf['table']['prefix'].$conf['table']['campaigns'] => 'm', $conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if ($includeStats) $aTables += array($conf['table']['prefix'].$conf['table']['data_summary_ad_hourly'] => 's', $conf['table']['prefix'].$conf['table']['banners'] => 'd', $conf['table']['prefix'].$conf['table']['campaigns'] => 'm');
            break;
        
        case 'ad_category_assoc' :
            $aTables += array($conf['table']['prefix'].$conf['table']['ad_category_assoc'] => 'ac');
            if (!empty($aParams['agency_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd', $conf['table']['campaigns'] => 'm', $conf['table']['prefix'].$conf['table']['clients'] = 'a');
            if (!empty($aParams['advertiser_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd', $conf['table']['prefix'].$conf['table']['campaigns'] => 'm');
            if (!empty($aParams['placement_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['placement_active'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd', $conf['table']['prefix'].$conf['table']['campaigns'] => 'm');
            if (!empty($aParams['ad_width'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['ad_height'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['ad_type'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['ad_active'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            break;
        
        case 'ad_zone_assoc' :
            $aTables += array($conf['table']['prefix'].$conf['table']['ad_zone_assoc'] => 'az');
            if (!empty($aParams['agency_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['zones'] => 'z', $conf['table']['prefix'].$conf['table']['banners'] => 'd', $conf['table']['prefix'].$conf['table']['campaigns'] => 'm', $conf['table']['prefix'].$conf['table']['clients'] = 'a', $conf['table']['prefix'].$conf['table']['affiliates'] => 'p');
            if (!empty($aParams['publisher_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['zones'] => 'z');
            if (!empty($aParams['advertiser_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd', $conf['table']['prefix'].$conf['table']['campaigns'] => 'm');
            if (!empty($aParams['placement_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['placement_active'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd', $conf['table']['prefix'].$conf['table']['campaigns'] => 'm');
            if (!empty($aParams['ad_width'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['ad_height'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['ad_type'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['ad_active'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            break;
        
        case 'agency' :
            $aTables += array($conf['table']['prefix'].$conf['table']['agency'] => 'g');
            if (!empty($aParams['advertiser_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['clients'] => 'a');
            if (!empty($aParams['placement_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['clients'] => 'a', $conf['table']['prefix'].$conf['table']['campaigns'] => 'm');
            if (!empty($aParams['ad_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['clients'] => 'a', $conf['table']['prefix'].$conf['table']['campaigns'] => 'm', $conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['ad_width'])) $aTables += array($conf['table']['prefix'].$conf['table']['clients'] => 'a', $conf['table']['prefix'].$conf['table']['campaigns'] => 'm', $conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['ad_height'])) $aTables += array($conf['table']['prefix'].$conf['table']['clients'] => 'a', $conf['table']['prefix'].$conf['table']['campaigns'] => 'm', $conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['ad_type'])) $aTables += array($conf['table']['prefix'].$conf['table']['clients'] => 'a', $conf['table']['prefix'].$conf['table']['campaigns'] => 'm', $conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['publisher_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['affiliates'] => 'p');
            if (!empty($aParams['zone_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['affiliates'] => 'p', $conf['table']['prefix'].$conf['table']['zones'] => 'z');
            if ($includeStats) $aTables += array($conf['table']['prefix'].$conf['table']['data_summary_ad_hourly'] => 's', $conf['table']['prefix'].$conf['table']['banners'] => 'd', $conf['table']['prefix'].$conf['table']['campaigns'] => 'm', $conf['table']['prefix'].$conf['table']['clients'] => 'a', $conf['table']['prefix'].$conf['table']['affiliates'] => 'p', $conf['table']['prefix'].$conf['table']['zones'] => 'z');
            break;
        
        case 'category' :
            $aTables += array($conf['table']['prefix'].$conf['table']['category'] => 'cat');
            break;
        
        case 'image' :
            $aTables += array($conf['table']['prefix'].$conf['table']['images'] => 'i');
            break;
        
        case 'limitation' :
            $aTables += array($conf['table']['prefix'].$conf['table']['acls'] => 'l');
            if (!empty($aParams['agency_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd', $conf['table']['prefix'].$conf['table']['campaigns'] => 'm', $conf['table']['prefix'].$conf['table']['clients'] = 'a');
            if (!empty($aParams['advertiser_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd', $conf['table']['prefix'].$conf['table']['campaigns'] => 'm');
            if (!empty($aParams['placement_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            break;
        
        case 'placement' :
            $aTables += array($conf['table']['prefix'].$conf['table']['campaigns'] => 'm');
            if (!empty($aParams['agency_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['clients'] => 'a');
            if (!empty($aParams['ad_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['ad_active'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['ad_width'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['ad_height'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['ad_type'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if ($includeStats) $aTables += array($conf['table']['prefix'].$conf['table']['data_summary_ad_hourly'] => 's', $conf['table']['prefix'].$conf['table']['banners'] => 'd');
            break;
        
        case 'placement_tracker' :
            $aTables += array($conf['table']['prefix'].$conf['table']['campaigns_trackers'] => 'mt');
            if (!empty($aParams['agency_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['campaigns'] => 'm', $conf['table']['prefix'].$conf['table']['clients'] => 'a', $conf['table']['prefix'].$conf['table']['trackers'] => 't');
            if (!empty($aParams['advertiser_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['campaigns'] => 'm', $conf['table']['prefix'].$conf['table']['trackers'] => 't');
            break;
        
        case 'placement_zone_assoc' :
            $aTables += array($conf['table']['prefix'].$conf['table']['placement_zone_assoc'] => 'pz');
            if (!empty($aParams['agency_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['zones'] => 'z', $conf['table']['prefix'].$conf['table']['campaigns'] => 'm', $conf['table']['prefix'].$conf['table']['clients'] => 'a', $conf['table']['prefix'].$conf['table']['affiliates'] => 'p');
            if (!empty($aParams['publisher_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['zones'] => 'z');
            if (!empty($aParams['advertiser_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['campaigns'] => 'm');
            if (!empty($aParams['ad_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            break;
        
        case 'publisher' :
            $aTables += array($conf['table']['prefix'].$conf['table']['affiliates'] => 'p');
            if (!empty($aParams['zone_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['zones'] => 'z');
            if ($includeStats) {
                $aTables += array($conf['table']['prefix'].$conf['table']['data_summary_ad_hourly'] => 's', $conf['table']['prefix'].$conf['table']['zones'] => 'z');
                if (!empty($aParams['placement_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            }
            break;
        
        case 'stats' :
            $aTables += array($conf['table']['prefix'].$conf['table']['data_summary_ad_hourly'] => 's');
            if (!empty($aParams['agency_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['clients'] => 'a', $conf['table']['prefix'].$conf['table']['campaigns'] => 'm', $conf['table']['prefix'].$conf['table']['banners'] => 'd', $conf['table']['prefix'].$conf['table']['affiliates'] => 'p', $conf['table']['prefix'].$conf['table']['zones'] => 'z');
            if (!empty($aParams['advertiser_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['campaigns'] => 'm', $conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['placement_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            if (!empty($aParams['publisher_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['zones'] => 'z');
            break;
        
        case 'tracker' :
            $aTables += array($conf['table']['prefix'].$conf['table']['trackers'] => 't');
            if (!empty($aParams['agency_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['clients'] => 'a');
            break;
        
        case 'variable' :
            $aTables += array($conf['table']['prefix'].$conf['table']['variables'] => 'v');
            if (!empty($aParams['agency_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['clients'] => 'a', $conf['table']['prefix'].$conf['table']['trackers'] => 't');
            if (!empty($aParams['advertiser_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['trackers'] => 't');
            break;
        
        case 'zone' :
            $aTables += array($conf['table']['prefix'].$conf['table']['zones'] => 'z');
            if (!empty($aParams['agency_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['affiliates'] => 'p');
            if ($includeStats) $aTables += array($conf['table']['prefix'].$conf['table']['data_summary_ad_hourly'] => 's');
            if ($includeStats) {
                $aTables += array($conf['table']['prefix'].$conf['table']['data_summary_ad_hourly'] => 's');
                if (!empty($aParams['placement_id'])) $aTables += array($conf['table']['prefix'].$conf['table']['banners'] => 'd');
            }
            break;
        }
        return $aTables;
    }
    
    /**
     * Returns an table name => alias pair.
     *
     * The output of this method is identical the _getTables() above, providing
     * the second parameter, $aParams, is an empty array.
     *
     * @param string $entity
     * @return array    An array of the prefixed table name and table alias
     */
    function _getPrimaryTable($entity)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
    
        $aTable = '';
        switch ($entity) {
        
        case 'ad' :                     $aTable = array($conf['table']['prefix'].$conf['table']['banners'] => 'd'); break;
        case 'advertiser' :             $aTable = array($conf['table']['prefix'].$conf['table']['clients'] => 'a'); break;
        case 'ad_category_assoc' :      $aTable = array($conf['table']['prefix'].$conf['table']['ad_category_assoc'] => 'ac'); break;
        case 'ad_zone_assoc' :          $aTable = array($conf['table']['prefix'].$conf['table']['ad_zone_assoc'] => 'az'); break;
        case 'agency' :                 $aTable = array($conf['table']['prefix'].$conf['table']['agency'] => 'g'); break;
        case 'category' :               $aTable = array($conf['table']['prefix'].$conf['table']['category'] => 'cat'); break;
        case 'image' :                  $aTable = array($conf['table']['prefix'].$conf['table']['images'] => 'i'); break;
        case 'limitation' :             $aTable = array($conf['table']['prefix'].$conf['table']['acls'] => 'l'); break;
        case 'placement' :              $aTable = array($conf['table']['prefix'].$conf['table']['campaigns'] => 'm'); break;
        case 'placement_tracker' :      $aTable = array($conf['table']['prefix'].$conf['table']['campaigns_trackers'] => 'mt'); break;
        case 'placement_zone_assoc' :   $aTable = array($conf['table']['prefix'].$conf['table']['placement_zone_assoc'] => 'pz'); break;
        case 'publisher' :              $aTable = array($conf['table']['prefix'].$conf['table']['affiliates'] => 'p'); break;
        case 'stats' :                  $aTable = array($conf['table']['prefix'].$conf['table']['data_summary_ad_hourly'] => 's'); break;
        case 'tracker' :                $aTable = array($conf['table']['prefix'].$conf['table']['trackers'] => 't'); break;
        case 'variable' :               $aTable = array($conf['table']['prefix'].$conf['table']['variables'] => 'v'); break;
        case 'zone' :                   $aTable = array($conf['table']['prefix'].$conf['table']['zones'] => 'z'); break;
        }
        return $aTable;
    }    
    
    /**
     * Returns an array of constraints given an entity with params.
     *
     * @param string $entity
     * @param array $aParams
     * @return array
     */
    function _getLimitations($entity, $aParams)
    {
        $aLimitations = array();
    
        if (!empty($aParams['ad_width'])) Admin_DA::_addLimitation($aLimitations, 'ad_width', 'd.width', $aParams['ad_width']);
        if (!empty($aParams['ad_height'])) Admin_DA::_addLimitation($aLimitations, 'ad_height', 'd.height', $aParams['ad_height']);
        if (!empty($aParams['ad_type'])) Admin_DA::_addLimitation($aLimitations, 'ad_type', 'd.storagetype', $aParams['ad_type']);
        if (!empty($aParams['ad_active'])) Admin_DA::_addLimitation($aLimitations, 'ad_active', 'd.active', $aParams['ad_active']);
        if (!empty($aParams['placement_active'])) Admin_DA::_addLimitation($aLimitations, 'placement_active', 'm.active', $aParams['placement_active']);
    
        switch ($entity) {
        
        case 'ad' :
            if (!empty($aParams['agency_id'])) Admin_DA::_addLimitation($aLimitations, 'agency_id', 'a.agencyid', $aParams['agency_id']);
            if (!empty($aParams['advertiser_id'])) Admin_DA::_addLimitation($aLimitations, 'advertiser_id', 'm.clientid', $aParams['advertiser_id']);
            if (!empty($aParams['placement_id'])) Admin_DA::_addLimitation($aLimitations, 'placement_id', 'd.campaignid', $aParams['placement_id']);
            if (!empty($aParams['ad_id'])) Admin_DA::_addLimitation($aLimitations, 'ad_id', 'd.bannerid', $aParams['ad_id']);
            break;
            
        case 'advertiser' :
            if (!empty($aParams['agency_id'])) Admin_DA::_addLimitation($aLimitations, 'agency_id', 'a.agencyid', $aParams['agency_id']);
            if (!empty($aParams['advertiser_id'])) Admin_DA::_addLimitation($aLimitations, 'advertiser_id', 'a.clientid', $aParams['advertiser_id']);
            if (!empty($aParams['placement_id'])) Admin_DA::_addLimitation($aLimitations, 'placement_id', 'm.campaignid', $aParams['placement_id']);
            if (!empty($aParams['ad_id'])) Admin_DA::_addLimitation($aLimitations, 'ad_id', 'd.bannerid', $aParams['ad_id']);
            break;
            
        case 'ad_category_assoc' :
            if (!empty($aParams['agency_id'])) Admin_DA::_addLimitation($aLimitations, 'agency_id', 'c.agencyid', $aParams['agency_id']);
            if (!empty($aParams['advertiser_id'])) Admin_DA::_addLimitation($aLimitations, 'advertiser_id', 'm.clientid', $aParams['advertiser_id']);
            if (!empty($aParams['placement_id'])) Admin_DA::_addLimitation($aLimitations, 'placement_id', 'd.campaignid', $aParams['placement_id']);
            if (!empty($aParams['ad_id'])) Admin_DA::_addLimitation($aLimitations, 'ad_id', 'ac.ad_id', $aParams['ad_id']);
            if (!empty($aParams['ad_category_assoc_id'])) Admin_DA::_addLimitation($aLimitations, 'ad_category_assoc_id', 'ac.ad_category_assoc_id', $aParams['ad_category_assoc_id']);
            break;
            
        case 'ad_zone_assoc' :
            if (!empty($aParams['agency_id'])) {
               Admin_DA::_addLimitation($aLimitations, 'agency_id', 'p.agencyid', $aParams['agency_id']);
               Admin_DA::_addLimitation($aLimitations, 'agency_id', 'c.agencyid', $aParams['agency_id']);
            }
            if (!empty($aParams['publisher_id'])) Admin_DA::_addLimitation($aLimitations, 'publisher_id', 'z.affiliateid', $aParams['publisher_id']);
            if (!empty($aParams['advertiser_id'])) Admin_DA::_addLimitation($aLimitations, 'advertiser_id', 'm.clientid', $aParams['advertiser_id']);
            if (!empty($aParams['zone_id'])) Admin_DA::_addLimitation($aLimitations, 'zone_id', 'az.zone_id', $aParams['zone_id']);
            if (!empty($aParams['placement_id'])) Admin_DA::_addLimitation($aLimitations, 'placement_id', 'd.campaignid', $aParams['placement_id']);
            if (!empty($aParams['ad_id'])) Admin_DA::_addLimitation($aLimitations, 'ad_id', 'az.ad_id', $aParams['ad_id']);
            if (!empty($aParams['ad_zone_assoc_id'])) Admin_DA::_addLimitation($aLimitations, 'ad_zone_assoc_id', 'az.ad_zone_assoc_id', $aParams['ad_zone_assoc_id']);
            break;
            
        case 'agency' :
            if (!empty($aParams['agency_id'])) Admin_DA::_addLimitation($aLimitations, 'agency_id', 'g.agencyid', $aParams['agency_id']);
            if (!empty($aParams['advertiser_id'])) Admin_DA::_addLimitation($aLimitations, 'advertiser_id', 'a.clientid', $aParams['advertiser_id']);
            if (!empty($aParams['placement_id'])) Admin_DA::_addLimitation($aLimitations, 'placement_id', 'm.campaignid', $aParams['placement_id']);
            if (!empty($aParams['ad_id'])) Admin_DA::_addLimitation($aLimitations, 'ad_id', 'd.bannerid', $aParams['ad_id']);
            if (!empty($aParams['publisher_id'])) Admin_DA::_addLimitation($aLimitations, 'publisher_id', 'p.affiliateid', $aParams['publisher_id']);
            if (!empty($aParams['zone_id'])) Admin_DA::_addLimitation($aLimitations, 'zone_id', 'z.zoneid', $aParams['zone_id']);
            break;
            
        case 'category' :
            if (!empty($aParams['name'])) Admin_DA::_addLimitation($aLimitations, 'name', 'cat.name', $aParams['name']);
            break;
            
        case 'image' :
            if (!empty($aParams['file_name'])) Admin_DA::_addLimitation($aLimitations, 'file_name', 'i.filename', $aParams['file_name']);
            break;
            
        case 'limitation' :
            if (!empty($aParams['agency_id'])) Admin_DA::_addLimitation($aLimitations, 'agency_id', 'c.agencyid', $aParams['agency_id']);
            if (!empty($aParams['advertiser_id'])) Admin_DA::_addLimitation($aLimitations, 'advertiser_id', 'm.clientid', $aParams['advertiser_id']);
            if (!empty($aParams['placement_id'])) Admin_DA::_addLimitation($aLimitations, 'placement_id', 'd.campaignid', $aParams['placement_id']);
            if (!empty($aParams['ad_id'])) Admin_DA::_addLimitation($aLimitations, 'ad_id', 'l.bannerid', $aParams['ad_id']);
            break;
            
        case 'placement' :
            if (!empty($aParams['agency_id'])) Admin_DA::_addLimitation($aLimitations, 'agency_id', 'a.agencyid', $aParams['agency_id']);
            if (!empty($aParams['advertiser_id'])) Admin_DA::_addLimitation($aLimitations, 'advertiser_id', 'm.clientid', $aParams['advertiser_id']);
            if (!empty($aParams['placement_id'])) Admin_DA::_addLimitation($aLimitations, 'placement_id', 'm.campaignid', $aParams['placement_id']);
            if (!empty($aParams['ad_id'])) Admin_DA::_addLimitation($aLimitations, 'ad_id', 'd.bannerid', $aParams['ad_id']);
            break;
            
        case 'placement_tracker' :
            if (!empty($aParams['agency_id'])) Admin_DA::_addLimitation($aLimitations, 'agency_id', 'c.agencyid', $aParams['agency_id']);
            if (!empty($aParams['advertiser_id'])) {
               Admin_DA::_addLimitation($aLimitations, 'advertiser_id', 'm.clientid', $aParams['advertiser_id']);
               Admin_DA::_addLimitation($aLimitations, 'advertiser_id', 't.clientid', $aParams['advertiser_id']);
            }
            if (!empty($aParams['tracker_id'])) Admin_DA::_addLimitation($aLimitations, 'tracker_id', 'mt.trackerid', $aParams['tracker_id']);
            if (!empty($aParams['placement_id'])) Admin_DA::_addLimitation($aLimitations, 'placement_id', 'mt.campaignid', $aParams['placement_id']);
            break;
            
        case 'placement_zone_assoc' :
            if (!empty($aParams['agency_id'])) {
               Admin_DA::_addLimitation($aLimitations, 'agency_id', 'p.agencyid', $aParams['agency_id']);
               Admin_DA::_addLimitation($aLimitations, 'agency_id', 'c.agencyid', $aParams['agency_id']);
            }
            if (!empty($aParams['publisher_id'])) Admin_DA::_addLimitation($aLimitations, 'publisher_id', 'z.affiliateid', $aParams['publisher_id']);
            if (!empty($aParams['advertiser_id'])) Admin_DA::_addLimitation($aLimitations, 'advertiser_id', 'm.clientid', $aParams['advertiser_id']);
            if (!empty($aParams['zone_id'])) Admin_DA::_addLimitation($aLimitations, 'zone_id', 'pz.zone_id', $aParams['zone_id']);
            if (!empty($aParams['placement_id'])) Admin_DA::_addLimitation($aLimitations, 'placement_id', 'pz.placement_id', $aParams['placement_id']);
            if (!empty($aParams['ad_id'])) Admin_DA::_addLimitation($aLimitations, 'ad_id', 'd.bannerid', $aParams['ad_id']);
            if (!empty($aParams['placement_zone_assoc_id'])) Admin_DA::_addLimitation($aLimitations, 'placement_zone_assoc_id', 'pz.placement_zone_assoc_id', $aParams['placement_zone_assoc_id']);
            break;
            
        case 'publisher' :
            if (!empty($aParams['agency_id'])) Admin_DA::_addLimitation($aLimitations, 'agency_id', 'p.agencyid', $aParams['agency_id']);
            if (!empty($aParams['publisher_id'])) Admin_DA::_addLimitation($aLimitations, 'publisher_id', 'p.affiliateid', $aParams['publisher_id']);
            if (!empty($aParams['zone_id'])) Admin_DA::_addLimitation($aLimitations, 'zone_id', 'z.zoneid', $aParams['zone_id']);
            break;
            
        case 'stats' :
            if (!empty($aParams['agency_id'])) {
               Admin_DA::_addLimitation($aLimitations, 'agency_id', 'p.agencyid', $aParams['agency_id']);
               Admin_DA::_addLimitation($aLimitations, 'agency_id', 'c.agencyid', $aParams['agency_id']);
            }
            if (!empty($aParams['publisher_id'])) Admin_DA::_addLimitation($aLimitations, 'publisher_id', 'z.affiliateid', $aParams['publisher_id']);
            if (!empty($aParams['advertiser_id'])) Admin_DA::_addLimitation($aLimitations, 'advertiser_id', 'm.clientid', $aParams['advertiser_id']);
            if (!empty($aParams['zone_id'])) Admin_DA::_addLimitation($aLimitations, 'zone_id', 's.zone_id', $aParams['zone_id']);
            if (!empty($aParams['placement_id'])) Admin_DA::_addLimitation($aLimitations, 'placement_id', 'd.campaignid', $aParams['placement_id']);
            if (!empty($aParams['ad_id'])) Admin_DA::_addLimitation($aLimitations, 'ad_id', 's.ad_id', $aParams['ad_id']);
            if (!empty($aParams['day_begin'])) $aLimitations[]="s.day>='{$aParams['day_begin']}'";
            if (!empty($aParams['day_end'])) $aLimitations[]="s.day<='{$aParams['day_end']}'";
            break;
            
        case 'tracker' :
            if (!empty($aParams['agency_id'])) Admin_DA::_addLimitation($aLimitations, 'agency_id', 'a.agencyid', $aParams['agency_id']);
            if (!empty($aParams['advertiser_id'])) Admin_DA::_addLimitation($aLimitations, 'advertiser_id', 't.clientid', $aParams['advertiser_id']);
            if (!empty($aParams['tracker_id'])) Admin_DA::_addLimitation($aLimitations, 'tracker_id', 't.trackerid', $aParams['tracker_id']);
            break;
            
        case 'variable' :
            if (!empty($aParams['agency_id'])) Admin_DA::_addLimitation($aLimitations, 'agency_id', 'a.agencyid', $aParams['agency_id']);
            if (!empty($aParams['advertiser_id'])) Admin_DA::_addLimitation($aLimitations, 'advertiser_id', 't.clientid', $aParams['advertiser_id']);
            if (!empty($aParams['tracker_id'])) Admin_DA::_addLimitation($aLimitations, 'tracker_id', 'v.trackerid', $aParams['tracker_id']);
            if (!empty($aParams['variable_id'])) Admin_DA::_addLimitation($aLimitations, 'variable_id', 'v.variableid', $aParams['variable_id']);
            break;
            
        case 'zone' :
            if (!empty($aParams['agency_id'])) Admin_DA::_addLimitation($aLimitations, 'agency_id', 'p.agencyid', $aParams['agency_id']);
            if (!empty($aParams['publisher_id'])) Admin_DA::_addLimitation($aLimitations, 'publisher_id', 'z.affiliateid', $aParams['publisher_id']);
            if (!empty($aParams['zone_id'])) Admin_DA::_addLimitation($aLimitations, 'zone_id', 'z.zoneid', $aParams['zone_id']);
            if (!empty($aParams['zone_type'])) Admin_DA::_addLimitation($aLimitations, 'zone_type', 'z.delivery', $aParams['zone_type']);
            if (!empty($aParams['zone_width'])) Admin_DA::_addLimitation($aLimitations, 'zone_width', 'z.width', $aParams['zone_width']);
            if (!empty($aParams['zone_height'])) Admin_DA::_addLimitation($aLimitations, 'zone_height', 'z.height', $aParams['zone_height']);
            break;
        }
        return $aLimitations;
    }
    
    /**
     * Builds the limitations array.
     *
     * @param array $aLimitations
     * @param string $entityIdName
     * @param string $columnName
     * @param string $value
     */
    function _addLimitation(&$aLimitations, $entityIdName, $columnName, $value)
    {
        // Add single quotes around non-integer columns
        if (       ($entityIdName == 'ad_type')
                || ($entityIdName == 'ad_active') 
                || ($entityIdName == 'placement_active')) {
            $value = "'" . str_replace(',', "','", $value) . "'";
        }
    
        // If there are multiple values, use IN instead of =
        if (strpos($value, ',') !== false) {
            $aLimitations[] = "$columnName IN ($value)";
        } else {
            $aLimitations[] = "$columnName=$value";
        }
    }    
    
    /**
     * Returns an array of constraints given an entity with params.
     *
     * @param string $entity
     * @param array $aParams
     * @return array
     */
    function _getStatsLimitations($entity, $aParams)
    {
        $aLimitations = array();
    
        if (!empty($aParams['agency_id'])) {
            switch ($entity) {
            
            case 'advertiser' :
            case 'placement' :
            case 'ad' :
                $aLimitations[] = "a.agencyid={$aParams['agency_id']}";
                break;
            case 'publisher' :
            case 'zone' :
                $aLimitations[] = "p.agencyid={$aParams['agency_id']}";
                break;
            default :
                $aLimitations[] = "p.agencyid={$aParams['agency_id']}";
                $aLimitations[] = "a.agencyid={$aParams['agency_id']}";
            }
        }
    
        if (!empty($aParams['publisher_id'])) Admin_DA::_addLimitation($aLimitations, 'publisher_id', 'z.affiliateid', $aParams['publisher_id']);
        if (!empty($aParams['advertiser_id'])) Admin_DA::_addLimitation($aLimitations, 'advertiser_id', 'm.clientid', $aParams['advertiser_id']);
        if (!empty($aParams['zone_id'])) Admin_DA::_addLimitation($aLimitations, 'zone_id', 's.zone_id', $aParams['zone_id']);
        if (!empty($aParams['placement_id'])) Admin_DA::_addLimitation($aLimitations, 'placement_id', 'd.campaignid', $aParams['placement_id']);
        if (!empty($aParams['ad_id'])) Admin_DA::_addLimitation($aLimitations, 'ad_id', 's.ad_id', $aParams['ad_id']);
        if (!empty($aParams['day_begin'])) $aLimitations[]="s.day>='{$aParams['day_begin']}'";
        if (!empty($aParams['day_end'])) $aLimitations[]="s.day<='{$aParams['day_end']}'";
    
        return $aLimitations;
    }
    
    /**
     * Builds an array of contraints given a list of tables.
     *
     * @param array $aTables
     * @return array
     */
    function _getTableLimitations($aTables)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $aTableLimitations = array();
        if (!empty($aTables[$conf['table']['prefix'].$conf['table']['agency']]) && (!empty($aTables[$conf['table']['prefix'].$conf['table']['clients']]))) 
            $aTableLimitations[]='g.agencyid=a.agencyid';
        if (!empty($aTables[$conf['table']['prefix'].$conf['table']['agency']]) && (!empty($aTables[$conf['table']['prefix'].$conf['table']['affiliates']]))) 
            $aTableLimitations[]='g.agencyid=p.agencyid';
        if (!empty($aTables[$conf['table']['prefix'].$conf['table']['affiliates']]) && (!empty($aTables[$conf['table']['prefix'].$conf['table']['zones']]))) 
            $aTableLimitations[]='p.affiliateid=z.affiliateid';
        if (!empty($aTables[$conf['table']['prefix'].$conf['table']['clients']]) && (!empty($aTables[$conf['table']['prefix'].$conf['table']['campaigns']]))) 
            $aTableLimitations[]='a.clientid=m.clientid';
        if (!empty($aTables[$conf['table']['prefix'].$conf['table']['campaigns']]) && (!empty($aTables[$conf['table']['prefix'].$conf['table']['banners']]))) 
            $aTableLimitations[]='m.campaignid=d.campaignid';
        if (!empty($aTables[$conf['table']['prefix'].$conf['table']['campaigns']]) && (!empty($aTables[$conf['table']['prefix'].$conf['table']['placement_zone_assoc']]))) 
            $aTableLimitations[]='m.campaignid=pz.placement_id';
        if (!empty($aTables[$conf['table']['prefix'].$conf['table']['campaigns']]) && (!empty($aTables[$conf['table']['prefix'].$conf['table']['campaigns_trackers']]))) 
            $aTableLimitations[]='m.campaignid=mt.campaignid';
        if (!empty($aTables[$conf['table']['prefix'].$conf['table']['trackers']]) && (!empty($aTables[$conf['table']['prefix'].$conf['table']['campaigns_trackers']]))) 
            $aTableLimitations[]='t.trackerid=mt.trackerid';
        if (!empty($aTables[$conf['table']['prefix'].$conf['table']['banners']]) && (!empty($aTables[$conf['table']['prefix'].$conf['table']['data_summary_ad_hourly']]))) 
            $aTableLimitations[]='d.bannerid=s.ad_id';
        if (!empty($aTables[$conf['table']['prefix'].$conf['table']['banners']]) && (!empty($aTables[$conf['table']['prefix'].$conf['table']['ad_category_assoc']]))) 
            $aTableLimitations[]='d.bannerid=ac.ad_id';
        if (!empty($aTables[$conf['table']['prefix'].$conf['table']['banners']]) && (!empty($aTables[$conf['table']['prefix'].$conf['table']['ad_zone_assoc']]))) 
            $aTableLimitations[]='d.bannerid=az.ad_id';
        if (!empty($aTables[$conf['table']['prefix'].$conf['table']['banners']]) && (!empty($aTables[$conf['table']['prefix'].$conf['table']['acls']]))) 
            $aTableLimitations[]='d.bannerid=l.bannerid';
        if (!empty($aTables[$conf['table']['prefix'].$conf['table']['zones']]) && (!empty($aTables[$conf['table']['prefix'].$conf['table']['data_summary_ad_hourly']]))) 
            $aTableLimitations[]='z.zoneid=s.zone_id';
        return $aTableLimitations;
    }
    
    
    
    
    
    
    
    // +---------------------------------------+
    // | SQL operations methods                |
    // |                                       |
    // | for entity builders                   |
    // +---------------------------------------+    
    
    /**
     * Performs an SQL delete.
     *
     * @param array $aTable
     * @param array $aLimitations
     * @param array $aOtherTables
     * @return integer  The number of rows affected by the delete
     */
    function _delete($aTable, $aLimitations, $aOtherTables = null)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $table = '';
        if (is_array($aTable)) {
            foreach ($aTable as $tableName => $alias) {
                $table = $tableName;
            }
        }
        //  check for prefix
        if (stristr($table, $conf['table']['prefix']) === false) {
            $table = $conf['table']['prefix'] . $table;
        }
    
        $otherTables = '';
        if (is_array($aOtherTables)) {
            foreach ($aOtherTables as $otherTable => $alias) {
                $otherTables .= ($otherTables == '') ? " USING $otherTable AS $alias" : ",$otherTable AS $alias";
            }
        }
    
        $where = '';
        if (is_array($aLimitations)) {
            foreach ($aLimitations as $limitation) {
                $where .= ($where == '') ? " WHERE $limitation" : " AND $limitation";
            }
        }
    
        $query = "DELETE FROM $table" . $otherTables . $where;
        $queryValid = true;

        // Doublecheck that there is something in the where clause
        //  - to ensure that a bug does not delete the entire contents of a table!
        if (strlen($where) > 0) {

            $dbh =& MAX_DB::singleton();
            $res = $dbh->query($query);
#            $res = phpAds_dbQuery($query)
#            or $queryValid = false;
        } else {
            $res = PEAR::raiseError('invalid WHERE clause');
        }
    
#        if ($queryValid) {
#            return phpAds_dbAffectedRows();
#        } else {
#            print_r($query . "\n\n" .mysql_error());
#            return false;
#        }
        if (!(PEAR::isError($res))) {
            $res = $dbh->affectedRows();
        }
        return $res;
    }
    
    /**
     * Performs an SQL insert.
     *
     * @param array $aTable
     * @param array $aVariables
     * @return integer  The number of rows affected by the insert
     */
    function _insert($aTable, $aVariables)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $table = '';
        if (is_array($aTable)) {
            foreach ($aTable as $tableName => $alias) {
                $table = $tableName;
            }
        }
        //  check for prefix
        if (stristr($table, $conf['table']['prefix']) === false) {
            $table = $conf['table']['prefix'] . $table;
        }
        
        $names = '';
        $values = '';
        foreach ($aVariables as $name => $value) {
            $names .= ($names == '') ? " ($name" : ",$name";
            $values .= ($values == '') ? " VALUES ('$value'" : ",'$value'";
        }
        if (!empty($names)) $names .= ')';
        if (!empty($values)) $values .= ')';
    
        $query = "INSERT INTO $table" . $names . $values;        
        $queryValid = true;

        $dbh =& MAX_DB::singleton();
        $res = $dbh->query($query);

        if (!(PEAR::isError($res))) {
            $res = MAX_DB::getInsertId();
        }
        return $res;
        #$res = phpAds_dbQuery($query)
        #or $queryValid = false;
    
#        if ($queryValid) {
#            return phpAds_dbInsertID();
#        } else {
#            print_r($query . "\n\n" .mysql_error());
#            return false;
#        }
    }
    
    /**
     * Wraps an SQL query.
     *
     * @param array $aColumns
     * @param array $aTables
     * @param array $aLimitations
     * @param array $aGroupColumns
     * @param string $primaryKey
     * @return integer  The number of rows affected by the query
     */
    function _select($aColumns, $aTables, $aLimitations, $aGroupColumns, $primaryKey)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $columns = '';
        if (is_array($aColumns)) {
            foreach ($aColumns as $column => $alias) {
                $columns .= ($columns == '') ? "SELECT $column AS $alias" : ",$column AS $alias";
            }
        } else {
            $columns = "SELECT $aColumns";
        }
    
        $tables = '';
        if (is_array($aTables)) {
            foreach ($aTables as $table => $alias) {
                //  check for prefix
                if (stristr($table, $conf['table']['prefix']) === false) {
                    $table = $conf['table']['prefix'] . $table;
                }
                $tables .= ($tables == '') ? " FROM $table AS $alias" : ",$table AS $alias";
            }
        } else {
            $tables = "FROM $aTables";
        }
    
        $where = '';
        if (is_array($aLimitations)) {
            foreach ($aLimitations as $limitation) {
                $where .= ($where == '') ? " WHERE $limitation" : " AND $limitation";
            }
        } else {
            $where = " WHERE $aLimitations";
        }
    
        $group = '';
        if (is_array($aGroupColumns) && count($aGroupColumns) > 0) {
            $group = ' GROUP BY ' . implode(',', $aGroupColumns);
        }
    
        $query = $columns . $tables . $where . $group;
        return Admin_DA::_query($query, $primaryKey);
    }
    
    /**
     * Performs and SQL query.
     *
     * @param string $query
     * @param string $primaryKey
     * @return array    An array of entity records
     */
    function _query($query, $primaryKey)
    {
        $queryValid = true;
    
        $dbh =& MAX_DB::singleton();
        $res = $dbh->query($query);
        #$res = phpAds_dbQuery($query)
        #or $queryValid = false;

        if (!(PEAR::isError($res))) {
            $aRows = array();
            while ($res->fetchInto($row)) {
                $aRows[$row[$primaryKey]] = $row;
            }
            return $aRows;
        }
/*        
        if ($queryValid) {
            $aRows = array();
            if (!empty($primaryKey)) {
                while ($row = phpAds_dbFetchArray($res)) {
                    $aRows[$row[$primaryKey]] = $row;
                }
            } else {
                while ($row = phpAds_dbFetchArray($res)) {
                    $aRows[] = $row;
                }
            }
        } else {
            print_r($query . "\n\n" .mysql_error());
        }
    
        return $queryValid ? $aRows : false;
*/
        return $res;
    }
    
    /**
     * Performs an SQL update.
     *
     * @param array $aTable
     * @param array $aVariables
     * @param array $aLimitations
     * @return integer  The number of rows affected by the update
     */
    function _update($aTable, $aVariables, $aLimitations)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $table = '';
        if (is_array($aTable)) {
            foreach ($aTable as $tableName => $alias) {
                //  check for prefix
                if (stristr($tableName, $conf['table']['prefix']) === false) {
                    $tableName = $conf['table']['prefix'] . $tableName;
                }
                $table = "$tableName AS $alias";
            }
        }
    
        $set = '';
        foreach ($aVariables as $name => $value) {
            $set .= ($set == '') ? " SET $name='$value'" : ",$name='$value'";
        }
    
        $where = '';
        if (is_array($aLimitations)) {
            foreach ($aLimitations as $limitation) {
                $where .= ($where == '') ? " WHERE $limitation" : " AND $limitation";
            }
        }
    
        $query = "UPDATE $table" . $set . $where;
        $queryValid = true;

        $dbh =& MAX_DB::singleton();
        $res = $dbh->query($query);        
        
        if (!(PEAR::isError($res))) {
            $res = $dbh->affectedRows();
        }
        return $res;
                
#        $res = phpAds_dbQuery($query)
#        or $queryValid = false;
#        if ($queryValid) {
#            return phpAds_dbAffectedRows();
#        } else {
#            print_r($query . "\n\n" .mysql_error());
#            return false;
#        }
    }
    
#    function MAX_connectDB()
#    {
#        if (empty($GLOBALS['_MAX']['ADMIN_DB_LINK'])) {
#            require_once 'lib-db.inc.php';
#            phpAds_dbConnect();
#        }
#    }






    // +---------------------------------------+
    // | Helper methods                        |
    // |                                       |
    // +---------------------------------------+

    /**
     * Returns a unique name for an entity.
     *
     * @param array $aEntity
     * @param array $aOtherEntities
     * @param string $str               The string 'copy' localised
     */
    function _getUniqueName(&$aEntity, $aOtherEntities, $str)
    {
        $same = false;
        if (!isset($str)) {
            $str = 'copy';
        }
        $name = $aEntity['name'];
        foreach ($aOtherEntities as $aOtherEntity) {
            if ($name == $aOtherEntity['name']) {
                $same = true;
                // Three cases:
                // a. 'joe' => 'joe (copy)'
                // b. 'joe (copy)' => 'joe (copy 2)'
                // c. 'joe (copy n)' => 'joe (copy n+1)'
                if (preg_match("/(.*)\($str (\d+)\)/", $name, $matches)) {
                    $n = $matches[2] + 1;
                    $name = "{$matches[1]}($str $n)";
                } elseif (preg_match("/(.*)\($str\)/", $name, $matches)) {
                    $name = "{$matches[1]}($str 2)";
                } else {
                    $name = "$name ($str)";
                }
                $aEntity['name'] = $name;
                break;
            }
        }
        if ($same) {
            Admin_DA::_getUniqueName($aEntity, $aOtherEntities, $str);
        }
    }

    /**
     * Swaps the field name with legacy table name.
     *
     * @param array $aParams    A hash of values to be switched
     * @param string $name
     * @param string $legacyName
     */
    function _switch(&$aParams, $name, $legacyName)
    {
        if (isset($aParams[$name])) {
            $aParams[$legacyName] = $aParams[$name];
            unset($aParams[$name]);
        }
    }    
    
    
    
    
    
    
    ////////////////////////////////////////////
    // +---------------------------------------+
    // | public methods                        |
    // |                                       |
    // | at last!                              |
    // +---------------------------------------+
    ////////////////////////////////////////////    
    
    
    
    
    
    
    

    // +---------------------------------------+
    // | stats methods                         |
    // |                                       |
    // | All methods related to gathering      |
    // | statistics                            |
    // +---------------------------------------+
    
    // old name: MAX_getCachedAdvertisersStats, not used anywhere
    /**
     * Returns a cached array of stats for an advertiser.
     *
     * @param array $aParams
     * @param boolean $allFields
     * @param integer $timeout
     * @return array    
     */
    function getCachedAdvertisersStats($aParams, $allFields=false, $timeout=-1)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
    
        include_once 'Cache/Lite/Function.php';
        $options = array('cacheDir' => MAX_CACHE, 'lifeTime' => ($timeout > -1 ? $timeout : $phpAds_config['cacheExpire']));
        $cache = new Cache_Lite_Function($options);
        return $cache->call('Admin_DA::_getAdvertisersStats', $aParams, $allFields);
    }
    
    // old name: MAX_getAdvertisersStats, not used anywhere
    /**
     * Returns an array of stats for an advertiser.
     *
     * @param array $aParams
     * @param boolean $allFields
     * @param integer $timeout
     * @return array    
     */
    function getAdvertisersStats($aParams, $allFields=false)
    {
        $aAdvertisers = Admin_DA::_getEntitiesChildren('advertiser', $aParams);
        $aAdvertisersStats = Admin_DA::_getEntitiesStats('advertiser', $aParams);
        $aActiveParams = array('placement_active' => 't', 'ad_active' => 't');
        $aActiveAdvertisers = Admin_DA::_getEntities('advertiser', $aParams + $aActiveParams);
        foreach ($aAdvertisers as $advertiserId => $aAdvertiser) {
            foreach (array('sum_requests', 'sum_views', 'sum_clicks', 'sum_conversions') as $item) {
                $aAdvertisers[$advertiserId][$item] = !empty($aAdvertisersStats[$advertiserId][$item]) ? $aAdvertisersStats[$advertiserId][$item] : 0;
            }
            $aAdvertisers[$advertiserId]['active'] = isset($aActiveAdvertisers[$advertiserId]);
        }
        return $aAdvertisers;
    }
    
    // old name: MAX_getCachedPlacementsStats        
    function getCachedPlacementsStats($aParams, $allFields=false, $timeout=-1)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
    
        include_once 'Cache/Lite/Function.php';
        $options = array('cacheDir' => MAX_CACHE, 'lifeTime' => ($timeout > -1 ? $timeout : $phpAds_config['cacheExpire']));
        $cache = new Cache_Lite_Function($options);
        return $cache->call('Admin_DA::_getPlacementsStats', $aParams, $allFields);
    }
    
    // old name: MAX_getPlacementsStats
    function getPlacementsStats($aParams, $allFields=false)
    {
        $aPlacements = Admin_DA::_getEntitiesChildren('placement', $aParams);
        $aPlacementsStats = Admin_DA::_getEntitiesStats('placement', $aParams);
        foreach ($aPlacements as $placementId => $aPlacement) {
            foreach (array('sum_requests', 'sum_views', 'sum_clicks', 'sum_conversions') as $item) {
                $aPlacements[$placementId][$item] = (!empty($aPlacementsStats[$placementId][$item])) 
                    ? $aPlacementsStats[$placementId][$item] 
                    : 0;
            }
        }
        return $aPlacements;
    }
    
    // old name: MAX_getCachedPublishersStats    
    function getCachedPublishersStats($aParams, $allFields=false, $timeout=-1)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
    
        include_once 'Cache/Lite/Function.php';
        $options = array('cacheDir' => MAX_CACHE, 'lifeTime' => ($timeout > -1 ? $timeout : $phpAds_config['cacheExpire']));
        $cache = new Cache_Lite_Function($options);
        return $cache->call('Admin_DA::_getPublishersStats', $aParams, $allFields);
    }
    
    // old name: MAX_getPublishersStats
    function getPublishersStats($aParams, $allFields=false)
    {
        $aPublishers = Admin_DA::_getEntitiesChildren('publisher', $aParams);
        $aPublishersStats = Admin_DA::_getEntitiesStats('publisher', $aParams);
        foreach ($aPublishers as $publisherId => $aPublisher) {
            foreach (array('sum_requests', 'sum_views', 'sum_clicks', 'sum_conversions') as $item) {
                $aPublishers[$publisherId][$item] = !empty($aPublishersStats[$publisherId][$item]) ? $aPublishersStats[$publisherId][$item] : 0;
            }
        }
    
        return $aPublishers;
    }

    // old name: MAX_getCachedZonesStats
    function getCachedZonesStats($aParams, $allFields=false, $timeout=-1)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
    
        include_once 'Cache/Lite/Function.php';
        $options = array('cacheDir' => MAX_CACHE, 'lifeTime' => ($timeout > -1 ? $timeout : $phpAds_config['cacheExpire']));
        $cache = new Cache_Lite_Function($options);
        return $cache->call('Admin_DA::_getZonesStats', $aParams, $allFields);
    }
    
    // old name: MAX_getZonesStats, not used anywhere
    function getZonesStats($aParams, $allFields=false)
    {
        $aZones = Admin_DA::_getEntitiesChildren('zone', $aParams);
        $aZonesStats = Admin_DA::_getEntitiesStats('zone', $aParams);
        foreach ($aZones as $zoneId => $aZone) {
            foreach (array('sum_requests', 'sum_views', 'sum_clicks', 'sum_conversions') as $item) {
                $aZones[$zoneId][$item] = !empty($aZonesStats[$zoneId][$item]) ? $aZonesStats[$zoneId][$item] : 0;
            }
        }
        return $aZones;
    }
    
    // old name: MAX_getCachedAdsStats
    function getCachedAdsStats($aParams, $allFields=false, $timeout=-1)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
    
        include_once 'Cache/Lite/Function.php';
        $options = array('cacheDir' => MAX_CACHE, 'lifeTime' => ($timeout > -1 ? $timeout : $phpAds_config['cacheExpire']));
        $cache = new Cache_Lite_Function($options);
        return $cache->call('Admin_DA::_getAdsStats', $aParams, $allFields);
    }
    
    // old name: MAX_getAdsStats
    function getAdsStats($aParams, $allFields=false)
    {
        $aAds = Admin_DA::_getEntities('ad', $aParams);
        $aAdsStats = Admin_DA::_getEntitiesStats('ad', $aParams);
        foreach ($aAds as $adId => $aAd) {
            foreach (array('sum_requests', 'sum_views', 'sum_clicks', 'sum_conversions') as $item) {
                $aAds[$adId][$item] = !empty($aAdsStats[$adId][$item]) ? $aAdsStats[$adId][$item] : 0;
            }
        }
        return $aAds;
    }
    
    
    
    // +---------------------------------------+
    // | generic DA methods                    |
    // |                                       |
    // +---------------------------------------+
    
    //  upgrade
    // old name: MAX_addCategory     
    function addCategory($aVariables)
    {
        return Admin_DA::_addEntity('category', $aVariables);
    }
    
    //  upgrade
    // old name: MAX_addAdCategory    
    function addAdCategory($aVariables)
    {
        return Admin_DA::_addEntity('ad_category_assoc', $aVariables);
    }
    
    //  upgrade, banner-zone
    // old name: MAX_addAdZone    
    function addAdZone($aVariables)
    {
        return Admin_DA::_addEntity('ad_zone_assoc', $aVariables);
    }
    
    function duplicateAdZone($aAdZone)
    {
        unset($aAdZone['zone_ad_id']);
        return Admin_DA::_addEntity('ad_zone_assoc', $aAdZone);
    }
    
    //  upgrade
    // old name: MAX_addPlacementZone    
    function addPlacementZone($aVariables)
    {
        return Admin_DA::_addEntity('placement_zone_assoc', $aVariables);
    }
    
    

    //  search and replace for all calls of the format MAX_getEntities('foo')
    function getAd($adId)
    {
        return Admin_DA::_getEntity('ad', $adId);
    }
   
    function getAds($aParams, $allFields = false)
    {
        return Admin_DA::_getEntities('ad', $aParams, $allFields);
    }
    
    function addAd($aParams)
    {
        return Admin_DA::_addEntity('ad', $aParams);
    }
    
    # old name: MAX_copyAd
    function duplicateAd($adId)
    {
        $aAd = Admin_DA::getAd($adId);
        return Admin_DA::_duplicateAd($aAd, true);
    }
    
    # old name: _copyAd
    function _duplicateAd($aAd, $checkUniqueNames = false)
    {
        require_once '../admin/lib-storage.inc.php';
    
        // Copy the linked creative
        $newFile = phpAds_ImageDuplicate($aAd['type'], $aAd['filename']);
        if ($newFile !== false) {
            $aAd['filename'] = $newFile;
        }
        // Remove the existing advert id
        $adId = $aAd['ad_id'];
        unset($aAd['ad_id']);

        // Check for unique names
        if ($checkUniqueNames) {
            $aAds = Admin_DA::getAds(array('placement_id' => $aAd['placement_id']));
#            $aAds = MAX_getEntities('ad', array('placement_id' => $aAd['placement_id']), false);
            Admin_DA::_getUniqueName($aAd, $aAds, $GLOBALS['strCopy']);
        }
        // Add the new ad
        $newAdId = Admin_DA::addAd('ad', $aAd);
    
        // Copy the limitations
        $aLimitations = Admin_DA::getLimitations(array('ad_id' => $adId));        
#        $aLimitations = MAX_getEntities('limitation', array('ad_id' => $adId));
        foreach ($aLimitations as $aLimitation) {
            $aLimitation['ad_id'] = $newAdId;
            Admin_DA::addLimitation($aLimitation);
        }
    
        // Copy the zone ad associations
        $aAdZones = Admin_DA::_getEntities('ad_zone_assoc', array('ad_id' => $adId));        
#        $aAdZones = MAX_getEntities('ad_zone_assoc', array('ad_id' => $adId));
        foreach ($aAdZones as $aAdZone) {
            $aAdZone['ad_id'] = $newAdId;
            Admin_DA::duplicateAdZone($aAdZone);
#            _copyAdZone($aAdZone);
        }
    
        return $newAdId;
    }    

    function getAdvertisers($aParams, $allFields = false)
    {
        return Admin_DA::_getEntities('advertiser', $aParams, $allFields);
    }

    function getAgency($agencyId)
    {
        return Admin_DA::_getEntity('agency', $agencyId);
    } 
    
    function getAgencies($aParams, $allFields = false)
    {
        return Admin_DA::_getEntities('agency', $aParams, $allFields);
    }
    
    function addAgency($aParams)
    {
        return Admin_DA::_addEntity('agency', $aParams);
    }      

    function addPlacement($aParams)
    {
        return Admin_DA::_addEntity('placement', $aParams);
    }
    
    function getPlacement($placementId)
    {
        return Admin_DA::_getEntity('placement', $placementId);
    }
    
    function getPlacements($aParams, $allFields = false)
    {
        return Admin_DA::_getEntities('placement', $aParams, $allFields);
    }
    
    #MAX_copyPlacement
    function duplicatePlacement($placementId)
    {
        $aPlacement = Admin_DA::getPlacement($placementId);
        return Admin_DA::_duplicatePlacement($aPlacement, true);
    }
    
    #_copyPlacement
    function _duplicatePlacement($aPlacement, $checkUniqueNames = false)
    {
        $placementId = $aPlacement['placement_id'];
        unset($aPlacement['placement_id']);
    
        if ($checkUniqueNames) {
            $aPlacements = Admin_DA::getPlacements(array('advertiser_id' => $aPlacement['advertiser_id']));
            Admin_DA::_getUniqueName($aPlacement, $aPlacements, $GLOBALS['strCopy']);
        }
        
        //  FIXME: get rid of this
        MAX_addslashes($aPlacement);
        //  massage to real field names
        Admin_DA::_switch($aPlacement, 'advertiser_id', 'clientid');
        Admin_DA::_switch($aPlacement, 'name', 'campaignname');
        $newPlacementId = Admin_DA::addPlacement($aPlacement);
    
        $aPlacementTrackers = Admin_DA::_getEntities('placement_tracker', array('placement_id' => $placementId));
        foreach ($aPlacementTrackers as $aPlacementTracker) {
            $aPlacementTracker['placement_id'] = $newPlacementId;
            Admin_DA::duplicatePlacementTracker($aPlacementTracker);
        }
    
        $aPlacementZones = Admin_DA::_getEntities('placement_zone_assoc', array('placement_id' => $placementId));
        foreach ($aPlacementZones as $aPlacementZone) {
            $aPlacementZone['placement_id'] = $newPlacementId;
            Admin_DA::duplicatePlacementZone($aPlacementZone);
        }
    
        $aAds = Admin_DA::getAds(array('placement_id' => $placementId));
        foreach ($aAds as $aAd) {
            $aAd['placement_id'] = $newPlacementId;
            Admin_DA::duplicateAd($aAd, $aAds);
        }
        return $newPlacementId;
    }
    
    function duplicatePlacementZone($aPlacementZone)
    {
        unset($aPlacementZone['placement_zone_assoc_id']);
        return Admin_DA::_addEntity('placement_zone_assoc', $aPlacementZone);
    }    
    
    #_copyPlacementTracker
    function duplicatePlacementTracker($aPlacementTracker)
    {
        unset($aPlacementTracker['placement_tracker_id']);
        return Admin_DA::_addEntity('placement_tracker', $aPlacementTracker);
    }    
    
    function getPublishers($aParams, $allFields = false)
    {
        return Admin_DA::_getEntities('publisher', $aParams, $allFields);
    }
        
    function getTracker($trackerId)
    {
        return Admin_DA::_getEntity('tracker', $trackerId);
    }    

    function getTrackers($aParams, $allFields = false)
    {
        return Admin_DA::_getEntities('tracker', $aParams, $allFields);
    }
    
    #MAX_copyTracker
    function duplicateTracker($trackerId)
    {
        $aTracker = Admin_DA::_getEntity('tracker', $trackerId);
        return Admin_DA::_duplicateTracker($aTracker, true);
    }   

    function addTracker($aParams)
    {
        return Admin_DA::_addEntity('tracker', $aParams);
    }
        
    #_copyTracker
    function _duplicateTracker($aTracker, $checkUniqueNames = false)
    {
        $trackerId = $aTracker['tracker_id'];
        unset($aTracker['tracker_id']);
    
        if ($checkUniqueNames) {
            $aTrackers = Admin_DA::getTrackers(array('advertiser_id' => $aTracker['advertiser_id']));        
#            $aTrackers = MAX_getEntities('tracker', array('advertiser_id' => $aTracker['advertiser_id']), false);
            Admin_DA::_getUniqueName($aTracker, $aTrackers, $GLOBALS['strCopy']);
        }
    
#        $newTrackerId = MAX_addEntity('tracker', $aTracker);
        //  massage to real field names
        Admin_DA::_switch($aTracker, 'advertiser_id', 'clientid');
        Admin_DA::_switch($aTracker, 'name', 'trackername');
        $newTrackerId = Admin_DA::addTracker($aTracker);
        if (PEAR::isError($newTrackerId)) {
            return PEAR::raiseError('failed to add tracker in ' . __FILE__ .','.__LINE__);
        }
    
        $aPlacementTrackers = Admin_DA::_getEntities('placement_tracker', array('tracker_id' => $trackerId));
        foreach ($aPlacementTrackers as $aPlacementTracker) {
            $aPlacementTracker['tracker_id'] = $newTrackerId;
            Admin_DA::duplicatePlacementTracker($aPlacementTracker);
        }
    
        $aVariables = Admin_DA::getVariables(array('tracker_id' => $trackerId));
        foreach ($aVariables as $aVariable) {
            $aVariable['tracker_id'] = $newTrackerId;
            Admin_DA::duplicateVariable($aVariable);
        }
        return $newTrackerId;
    }    

    function getZone($zoneId)
    {
        return Admin_DA::_getEntity('zone', $zoneId);
    }
    
    function getZones($aParams, $allFields = false)
    {
        return Admin_DA::_getEntities('zone', $aParams, $allFields);
    }
    
    #MAX_addEntity('zone')
    /*
    acceptable zones type IDs are:
        define ("phpAds_ZoneBanner", 0);
        define ("phpAds_ZoneInterstitial", 1);
        define ("phpAds_ZonePopup", 2);
        define ("phpAds_ZoneText", 3);    
    */
    function addZone($aVariables)
    {
        Admin_DA::_switch($aVariables, 'publisher_id', 'affiliateid');
        Admin_DA::_switch($aVariables, 'name', 'zonename');
        Admin_DA::_switch($aVariables, 'type', 'delivery');    
        return Admin_DA::_addEntity('zone', $aVariables);
    }
    
    #MAX_copyZone
    function duplicateZone($zoneId)
    {
        $aZone = Admin_DA::getZone($zoneId);
        return Admin_DA::_duplicateZone($aZone, true);
    }    
    
    # used to be called _copyZone
    function _duplicateZone($aZone, $checkUniqueNames = false)
    {
        $zoneId = $aZone['zone_id'];
        unset($aZone['zone_id']);
    
        if ($checkUniqueNames) {
            $aZones = Admin_DA::getZones(array('publisher_id' => $aZone['publisher_id']));   
    #        $aZones = MAX_getEntities('zone', array('publisher_id' => $aZone['publisher_id']), false);
            Admin_DA::_getUniqueName($aZone, $aZones, $GLOBALS['strCopy']);
    #       _getUniqueName($aZone, $aZones, $GLOBALS['strCopy']);        
        }
    
        $newZoneId = Admin_DA::addZone($aZone);
    #    $newZoneId = MAX_addEntity('zone', $aZone);
    
        //  FIXME
        $aPlacementZones = Admin_DA::_getEntities('placement_zone_assoc', array('zone_id' => $zoneId));
    #    $aPlacementZones = MAX_getEntities('placement_zone_assoc', array('zone_id' => $zoneId));
        foreach($aPlacementZones as $aPlacementZone) {
            $aPlacementZone['zone_id'] = $newZoneId;
            Admin_DA::duplicatePlacementZone($aPlacementZone);
    #        _copyPlacementZone($aPlacementZone);
        }
    
        $aAdZones = Admin_DA::_getEntities('ad_zone_assoc', array('zone_id' => $zoneId));
    #    $aAdZones = MAX_getEntities('ad_zone_assoc', array('zone_id' => $zoneId));
        foreach($aAdZones as $aAdZone) {
            $aAdZone['zone_id'] = $newZoneId;
            Admin_DA::duplicateAdZone($aAdZone);
    #        _copyAdZone($aAdZone);
        }
        return $newZoneId;
    }     
    
    function getLimitations($aParams, $allFields = false)
    {
        return Admin_DA::_getEntities('limitation', $aParams, $allFields);
    }
    
    function addLimitation($aVariable)
    {
        return Admin_DA::_addEntity('limitation', $aVariable);    
    }    

    function getVariables($aParams, $allFields = false)
    {
        return Admin_DA::_getEntities('variable', $aParams, $allFields);
    }
    
    function addVariable($aVariable)
    {
        return Admin_DA::_addEntity('variable', $aVariable);    
    }
    
    # used to be called _copyVariable()
    function duplicateVariable($aVariable)
    {
        unset($aVariable['variable_id']);
        return Admin_DA::addVariable($aVariable);
    }    
    
    function deleteImage($id)
    {
        return Admin_DA::_deleteEntity('image', $id);
    }    
    //  end: search and replace MAX_getEntities
}

?>