<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: TestOfDalStatisticsTrackermysql.php 2037 2005-03-11 14:52:59Z demian $
*/

/**
 * @package    MaxDal
 * @subpackage TestSuite
 * @author     Andrew Hill <andrew@awarez.net>
 */

require_once MAX_PATH . '/lib/Max.php';
require_once MAX_PATH . '/lib/max/DB.php';
require_once MAX_PATH . '/lib/max/Dal/Statistics/Tracker/mysql.php';

/**
 * A class for testing the MAX_Dal_Statistics_Tracker_mysql_MySql class.
 */
class Dal_TestOfMaxDalStatisticsTrackermysql extends UnitTestCase
{
    
    /**
     * The constructor method.
     */
    function Dal_TestOfMaxDalStatisticsTrackermysql()
    {
        $this->UnitTestCase();
    }
    
    /**
     * Tests that the singleton() method only ever returns one class instance.
     */
    function testSingleton()
    {
        MAX::debug('Running test', PEAR_LOG_INFO);
        $first = new MAX_Dal_Statistics_Tracker_mysql();
        $second = new MAX_Dal_Statistics_Tracker_mysql();
        $this->assertIdentical($first, $second);
    }
    
    /**
     * Tests the getMaintenanceStatisticsLastRunInfo() method.
     */
    function testGetMaintenanceStatisticsLastRunInfo()
    {
        MAX::debug('Running test', PEAR_LOG_INFO);
        // Use a reference to $GLOBALS['_MAX']['CONF'] so that the configuration
        // options can be changed while the test is running
        $conf =& $GLOBALS['_MAX']['CONF'];
        $dbh =& MAX_DB::singleton();
        $conf['maintenance']['operationInterval'] = 60;
        $conf['table']['split'] = false;
        $dsa = new MAX_Dal_Statistics_Tracker_mysql();
        // Create the required tables
        $dsa->tables->createTable('data_raw_tracker_impression');
        $dsa->tables->createTable('log_maintenance_statistics');
        // Test with no data
        $date = $dsa->getMaintenanceStatisticsLastRunInfo(DAL_STATISTICS_COMMON_UPDATE_OI);
        $this->assertNull($date);
        $date = $dsa->getMaintenanceStatisticsLastRunInfo(DAL_STATISTICS_COMMON_UPDATE_HOUR);
        $this->assertNull($date);
        // Get the data for the tests
        include_once MAX_PATH. '/lib/max/Dal/data/TestOfStatisticsTrackermysql.php';
        // Insert tracker impressions
        $result = $dbh->query(TOT_DATA_RAW_TRACKER_IMPRESSIONS);
        // Test
        $date = $dsa->getMaintenanceStatisticsLastRunInfo(DAL_STATISTICS_COMMON_UPDATE_OI);
        $this->assertEqual($date, new Date('2004-05-06 11:59:59'));
        $date = $dsa->getMaintenanceStatisticsLastRunInfo(DAL_STATISTICS_COMMON_UPDATE_HOUR);
        $this->assertEqual($date, new Date('2004-05-06 11:59:59'));
        // Insert an hourly (only) update
        $result = $dbh->query(TOT_LMS_HOUR);
        // Test
        $date = $dsa->getMaintenanceStatisticsLastRunInfo(DAL_STATISTICS_COMMON_UPDATE_OI);
        $this->assertEqual($date, new Date('2004-05-06 11:59:59'));
        $date = $dsa->getMaintenanceStatisticsLastRunInfo(DAL_STATISTICS_COMMON_UPDATE_HOUR);
        $this->assertEqual($date, new Date('2004-06-06 10:15:00'));
        // Insert an operation interval (only) update
        $result = $dbh->query(TOT_LMS_OI);
        // Test
        $date = $dsa->getMaintenanceStatisticsLastRunInfo(DAL_STATISTICS_COMMON_UPDATE_OI);
        $this->assertEqual($date, new Date('2004-06-06 10:16:00'));
        $date = $dsa->getMaintenanceStatisticsLastRunInfo(DAL_STATISTICS_COMMON_UPDATE_HOUR);
        $this->assertEqual($date, new Date('2004-06-06 10:15:00'));
        // Insert a dual interval update
        $result = $dbh->query(TOT_LMS_DUAL);
        // Test
        $date = $dsa->getMaintenanceStatisticsLastRunInfo(DAL_STATISTICS_COMMON_UPDATE_OI);
        $this->assertEqual($date, new Date('2004-06-07 01:15:00'));
        $date = $dsa->getMaintenanceStatisticsLastRunInfo(DAL_STATISTICS_COMMON_UPDATE_HOUR);
        $this->assertEqual($date, new Date('2004-06-07 01:15:00'));
        // Drop the tables
        $dsa->tables->dropTable($conf['table']['prefix'].$conf['table']['data_raw_tracker_impression']);
        $dsa->tables->dropTable($conf['table']['prefix'].$conf['table']['log_maintenance_statistics']);
    }
    
    /**
     * Tests the deleteOldData() method.
     */
    function testDeleteOldData()
    {
        MAX::debug('Running test', PEAR_LOG_INFO);
        // Use a reference to $GLOBALS['_MAX']['CONF'] so that the configuration
        // options can be changed while the test is running
        $conf =& $GLOBALS['_MAX']['CONF'];
        $dbh =& MAX_DB::singleton();
        $conf['maintenance']['compactStatsGrace'] = 0;
        $dsa = new MAX_Dal_Statistics_Tracker_mysql();
        // Create the required tables
        $dsa->tables->createTable('data_raw_tracker_click');
        $dsa->tables->createTable('data_raw_tracker_impression');
        $dsa->tables->createTable('data_raw_tracker_variable_value');
        // Get the data for the tests
        include_once maintenancePath . '/lib/max/Dal/data/TestOfStatisticsTrackermysql.php';
        // Insert the test data
        $result = $dbh->query(TOT_DELETE_OLD_DATA_TRACKER_CLICKS);
        $result = $dbh->query(TOT_DELETE_OLD_DATA_TRACKER_IMPRESSIONS);
        $result = $dbh->query(TOT_DELETE_OLD_DATA_TRACKER_VARIABLE_VALUES);
        // Test
        $summarisedTo = new Date('2004-06-06 17:59:59');
        $dsa->deleteOldData($summarisedTo);
        $query = "
            SELECT
                COUNT(*) AS number
            FROM
                {$conf['table']['prefix']}{$conf['table']['data_raw_tracker_click']}";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['number'], 1);
        $query = "
            SELECT
                COUNT(*) AS number
            FROM
                {$conf['table']['prefix']}{$conf['table']['data_raw_tracker_impression']}";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['number'], 1);
        $query = "
            SELECT
                COUNT(*) AS number
            FROM
                {$conf['table']['prefix']}{$conf['table']['data_raw_tracker_variable_value']}";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['number'], 1);
        // Set a compact_stats_grace window
        $conf['maintenance']['compactStatsGrace'] = 3600;
        $dsa = new MAX_Dal_Statistics_Tracker_mysql();
        // Drop and re-create the raw tables
        $dsa->tables->dropTable($conf['table']['prefix'].$conf['table']['data_raw_tracker_click']);
        $dsa->tables->dropTable($conf['table']['prefix'].$conf['table']['data_raw_tracker_impression']);
        $dsa->tables->dropTable($conf['table']['prefix'].$conf['table']['data_raw_tracker_variable_value']);
        $dsa->tables->createTable('data_raw_tracker_click');
        $dsa->tables->createTable('data_raw_tracker_impression');
        $dsa->tables->createTable('data_raw_tracker_variable_value');
        // Re-insert the test data
        $result = $dbh->query(TOT_DELETE_OLD_DATA_TRACKER_CLICKS);
        $result = $dbh->query(TOT_DELETE_OLD_DATA_TRACKER_IMPRESSIONS);
        $result = $dbh->query(TOT_DELETE_OLD_DATA_TRACKER_VARIABLE_VALUES);
        // Test
        $summarisedTo = new Date('2004-06-06 17:59:59');
        $dsa->deleteOldData($summarisedTo);
        $query = "
            SELECT
                COUNT(*) AS number
            FROM
                {$conf['table']['prefix']}{$conf['table']['data_raw_tracker_click']}";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['number'], 3);
        $query = "
            SELECT
                COUNT(*) AS number
            FROM
                {$conf['table']['prefix']}{$conf['table']['data_raw_tracker_impression']}";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['number'], 3);
        $query = "
            SELECT
                COUNT(*) AS number
            FROM
                {$conf['table']['prefix']}{$conf['table']['data_raw_tracker_variable_value']}";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['number'], 3);
    }

    /**
     * Tests the logCompletion() method.
     */
    function testLogCompletion()
    {
        MAX::debug('Running test', PEAR_LOG_INFO);
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh =& MAX_DB::singleton();
        $dsa = new MAX_Dal_Statistics_Tracker_mysql();
        // Create the required tables
        $dsa->tables->createTable('log_maintenance_statistics');
        $dsa->tables->createTable('userlog');
        // Test
        $start = new Date('2004-06-06 18:00:00');
        $end = new Date('2004-06-06 18:01:05');
        $report = 'Report Text';
        $dsa->logCompletion($start, $end, 0, $start, $report);
        $query = "
            SELECT
                COUNT(*) AS number
            FROM
                {$conf['table']['prefix']}{$conf['table']['log_maintenance_statistics']}
            WHERE
                tracker_run_type = 0";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['number'], 1);
        $query = "
            SELECT
                *
            FROM
                {$conf['table']['prefix']}{$conf['table']['log_maintenance_statistics']}
            WHERE
                tracker_run_type = 0";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['start_run'], '2004-06-06 18:00:00');
        $this->assertEqual($row['end_run'], '2004-06-06 18:01:05');
        $this->assertEqual($row['duration'], 65);
        $this->assertEqual($row['updated_to'], '2004-06-06 18:00:00');
        $start = new Date('2004-06-06 19:00:00');
        $end = new Date('2004-06-06 19:00:05');
        $dsa->logCompletion($start, $end, 1, $start, $report);
        $query = "
            SELECT
                COUNT(*) AS number
            FROM
                {$conf['table']['prefix']}{$conf['table']['log_maintenance_statistics']}
            WHERE
                tracker_run_type = 1";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['number'], 1);
        $query = "
            SELECT
                *
            FROM
                {$conf['table']['prefix']}{$conf['table']['log_maintenance_statistics']}
            WHERE
                tracker_run_type = 1";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['start_run'], '2004-06-06 19:00:00');
        $this->assertEqual($row['end_run'], '2004-06-06 19:00:05');
        $this->assertEqual($row['duration'], 5);
        $this->assertEqual($row['updated_to'], '2004-06-06 19:00:00');
        $start = new Date('2004-06-06 20:00:00');
        $end = new Date('2004-06-06 21:00:05');
        $dsa->logCompletion($start, $end, 2, $start, $report);
        $query = "
            SELECT
                COUNT(*) AS number
            FROM
                {$conf['table']['prefix']}{$conf['table']['log_maintenance_statistics']}
            WHERE
                tracker_run_type = 2";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['number'], 1);
        $query = "
            SELECT
                *
            FROM
                {$conf['table']['prefix']}{$conf['table']['log_maintenance_statistics']}
            WHERE
                tracker_run_type = 2";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['start_run'], '2004-06-06 20:00:00');
        $this->assertEqual($row['end_run'], '2004-06-06 21:00:05');
        $this->assertEqual($row['duration'], 3605);
        $this->assertEqual($row['updated_to'], '2004-06-06 20:00:00');
        // Drop the tables
        $dsa->tables->dropTable($conf['table']['prefix'].$conf['table']['log_maintenance_statistics']);
        $dsa->tables->dropTable($conf['table']['prefix'].$conf['table']['userlog']);
    }
    
}

?>
