<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| Copyright (c) 2000-2003 the phpAdsNew developers                          |
| For contact details, see: http://www.phpadsnew.com/                       |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: lib-limitations.inc.php 2024 2005-03-10 17:21:43Z andrew $
*/

/*-------------------------------------------------------*/
/* Check if the ACL is valid                             */
/*-------------------------------------------------------*/

function phpAds_aclCheck($row, $source)
{
    if (isset($row['compiledlimitation']) && ($row['compiledlimitation'] != '')) {
        // Set to true in case of error in eval
        $result = true;
        @eval('$result = ('.$row['compiledlimitation'].');');
        return $result;
    } else {
        return true;
    }
}

/*-------------------------------------------------------*/
/* Check if the Weekday ACL is valid                     */
/*-------------------------------------------------------*/

function phpAds_aclCheckWeekday($data, $ad)
{
	if ($data == '') {
		return true;
	}
	$day = date('w');
	$expression = ($data == "*" || $data == $day || in_array ($day, explode(',', $data)));
	$operator   = $ad == '==';
	return ($expression == $operator);
}

/*-------------------------------------------------------*/
/* Check if the Useragent ACL is valid                   */
/*-------------------------------------------------------*/

function phpAds_aclCheckUseragent($data, $ad)
{
    if ($data == '') {
        return true;
    }
    $agent = $_SERVER['HTTP_USER_AGENT'];
    $expression = ($data == "*" || preg_match('#'.$data.'#', $agent));
    $operator   = $ad == '==';
    return ($expression == $operator);
}

/*-------------------------------------------------------*/
/* Check if the Client IP ACL is valid                   */
/*-------------------------------------------------------*/

function phpAds_aclCheckClientip($data, $ad)
{
    if ($data == '') {
        return true;
    }
    $host = $_SERVER['REMOTE_ADDR'];
    if (!strpos($data, '/')) {
        $net = explode('.', $data);
        for ($i=0;$i<sizeof($net);$i++) {
            if ($net[$i] == '*') {
                $net[$i] = 0;
                $mask[$i] = 0;
            } else {
                $mask[$i] = 255;
            }
        }
        $pnet 	= pack('C4', $net[0], $net[1], $net[2], $net[3]);
        $pmask 	= pack('C4', $mask[0], $mask[1], $mask[2], $mask[3]);
    } else {
        list ($net, $mask) = explode('/', $data);
        $net 	= explode('.', $net);
        $pnet 	= pack('C4', $net[0], $net[1], $net[2], $net[3]);
        $mask 	= explode('.', $mask);
        $pmask 	= pack('C4', $mask[0], $mask[1], $mask[2], $mask[3]);
    }
    $host 	= explode('.', $host);
    $phost 	= pack('C4', $host[0], $host[1], $host[2], $host[3]);
    $expression = ($data == "*" || ($phost & $pmask) == $pnet);
    $operator   = $ad == '==';
    return ($expression == $operator);
}

/*-------------------------------------------------------*/
/* Check if the Domain ACL is valid                      */
/*-------------------------------------------------------*/

function phpAds_aclCheckDomain($data, $ad)
{
    if ($data == '') {
        return true;
    }
    $host = $_SERVER['REMOTE_HOST'];
    $domain 	= substr($host,-(strlen($data)));
    $expression = ($data == "*" || strtolower($domain) == strtolower($data)) ;
    $operator   = $ad == '==';
    return ($expression == $operator);
}

/*-------------------------------------------------------*/
/* Check if the Language ACL is valid                    */
/*-------------------------------------------------------*/

function phpAds_aclCheckLanguage($data, $ad)
{
    if ($data == '') {
        return true;
    }
    $source = $_SERVER['HTTP_ACCEPT_LANGUAGE'];
    $expression = ($data == "*" || preg_match('#^('.$data.')#', $source));
    $operator   = $ad == '==';
    return ($expression == $operator);
}

/*-------------------------------------------------------*/
/* Check if the Source ACL is valid                      */
/*-------------------------------------------------------*/

function phpAds_aclCheckSource($data, $ad, $source)
{
    if ($data == '') {
        return true;
    }
    $pattern = '#^'.str_replace('*', '.*', str_replace('.', '\.', str_replace('(', '\(', str_replace(')', '\)', $data)))).'$#i';
    $expression = ($data == "*" || strtolower($source) == strtolower($data) ||
    preg_match($pattern, $source));
    $operator   = $ad == '==';
    return ($expression == $operator);
}

/*-------------------------------------------------------*/
/* Check if the Time ACL is valid                        */
/*-------------------------------------------------------*/

function phpAds_aclCheckTime($data, $ad)
{
    if ($data == '') {
        return true;
    }
    $time = date('G');
    $expression = ($data == "*" || $data == $time || in_array ($time, explode(',', $data)));
    $operator   = $ad == '==';
    return ($expression == $operator);
}

/*-------------------------------------------------------*/
/* Check if the Date ACL is valid                        */
/*-------------------------------------------------------*/

function phpAds_aclCheckDate($data, $ad)
{
    if ($data == '' && $data == '00000000') {
        return true;
    }
    $date = date('Ymd');
    switch ($ad) {
        case '==':
        return ($date == $data); break;
        case '!=':
        return ($date != $data); break;
        case '<=':
        return ($date <= $data); break;
        case '>=':
        return ($date >= $data); break;
        case '<':
        return ($date < $data);  break;
        case '>':
        return ($date > $data);  break;
    }
    return true;
}

/**
 * A function to check Country Code ACLs.
 */
function phpAds_aclCheckCountryCode($data, $ad)
{
    global $geoInfo;
    if ($geoInfo && $geoInfo['country_code']) {
        // Evaluate country code
        $expression = ($data == $geoInfo['country_code'] || in_array($geoInfo['country_code'], explode(',', $data)));
        $operator   = $ad == '==';
        return ($expression == $operator);
    } else {
        return ($ad != '==');
    }
}

/**
 * A function to check Country Name ACLs.
 */
function phpAds_aclCheckCountryName($data, $ad)
{
    global $geoInfo;
    if ($geoInfo && $geoInfo['country_name']) {
        // Evaluate country name
        $expression = ($data == $geoInfo['country_name'] || in_array($geoInfo['country_name'], explode(',', $data)));
        $operator   = $ad == '==';
        return ($expression == $operator);
    } else {
        return ($ad != '==');
    }
}

/**
 * A function to check Region ACLs.
 */
function phpAds_aclCheckRegion($data, $ad)
{
    global $geoInfo;
    if ($geoInfo && $geoInfo['region']) {
        // Evaluate region code
        $expression = ($data == $geoInfo['region'] || in_array($geoInfo['region'], explode(',', $data)));
        $operator   = $ad == '==';
        return ($expression == $operator);
    } else {
        return ($ad != '==');
    }
}

/**
 * A function to check City ACLs.
 */
function phpAds_aclCheckCity($data, $ad)
{
    global $geoInfo;
    if ($geoInfo && $geoInfo['city']) {
        // Evaluate city code
        $expression = ($data == $geoInfo['city'] || in_array($geoInfo['city'], explode(',', $data)));
        $operator   = $ad == '==';
        return ($expression == $operator);
    } else {
        return ($ad != '==');
    }
}

/**
 * A function to check Postal Code ACLs.
 */
function phpAds_aclCheckPostalCode($data, $ad)
{
    global $geoInfo;
    if ($geoInfo && $geoInfo['postal_code']) {
        // Evaluate postal code code
        $expression = ($data == $geoInfo['postal_code'] || in_array($geoInfo['postal_code'], explode(',', $data)));
        $operator   = $ad == '==';
        return ($expression == $operator);
    } else {
        return ($ad != '==');
    }
}

/**
 * A function to check Latitude ACLs.
 */
function phpAds_aclCheckPostalLatitude($data, $ad)
{
    global $geoInfo;
    if ($geoInfo && $geoInfo['latitude']) {
        // Evaluate latitude code
        $expression = ($data == $geoInfo['latitude'] || in_array($geoInfo['latitude'], explode(',', $data)));
        $operator   = $ad == '==';
        return ($expression == $operator);
    } else {
        return ($ad != '==');
    }
}

/**
 * A function to check Longitude ACLs.
 */
function phpAds_aclCheckPostalLongitude($data, $ad)
{
    global $geoInfo;
    if ($geoInfo && $geoInfo['longitude']) {
        // Evaluate longitude code
        $expression = ($data == $geoInfo['longitude'] || in_array($geoInfo['longitude'], explode(',', $data)));
        $operator   = $ad == '==';
        return ($expression == $operator);
    } else {
        return ($ad != '==');
    }
}

/**
 * A function to check DMA Code ACLs.
 */
function phpAds_aclCheckDmaCode($data, $ad)
{
    global $geoInfo;
    if ($geoInfo && $geoInfo['dma_code']) {
        // Evaluate DMA code code
        $expression = ($data == $geoInfo['dma_code'] || in_array($geoInfo['dma_code'], explode(',', $data)));
        $operator   = $ad == '==';
        return ($expression == $operator);
    } else {
        return ($ad != '==');
    }
}

/**
 * A function to check Area Code ACLs.
 */
function phpAds_aclCheckAreaCode($data, $ad)
{
    global $geoInfo;
    if ($geoInfo && $geoInfo['area_code']) {
        // Evaluate Area code code
        $expression = ($data == $geoInfo['area_code'] || in_array($geoInfo['area_code'], explode(',', $data)));
        $operator   = $ad == '==';
        return ($expression == $operator);
    } else {
        return ($ad != '==');
    }
}

/**
 * A function to check Organisation ACLs.
 */
function phpAds_aclCheckAreaOrganisation($data, $ad)
{
    global $geoInfo;
    if ($geoInfo && $geoInfo['organisation']) {
        // Evaluate Organisation code
        $expression = ($data == $geoInfo['organisation'] || in_array($geoInfo['organisation'], explode(',', $data)));
        $operator   = $ad == '==';
        return ($expression == $operator);
    } else {
        return ($ad != '==');
    }
}

/**
 * A function to check Netspeed ACLs.
 */
function phpAds_aclCheckAreaNetspeed($data, $ad)
{
    global $geoInfo;
    if ($geoInfo && $geoInfo['netspeed']) {
        // Evaluate Netspeed code
        $expression = ($data == $geoInfo['netspeed'] || in_array($geoInfo['netspeed'], explode(',', $data)));
        $operator   = $ad == '==';
        return ($expression == $operator);
    } else {
        return ($ad != '==');
    }
}

/*-------------------------------------------------------*/
/* Check if the Continent ACL is valid                   */
/*-------------------------------------------------------*/

function phpAds_aclCheckContinent($data, $ad)
{
    global $geoInfo;
    if ($geoInfo && $geoInfo['continent']) {
        // Evaluate continent code
        $expression = ($data == $geoInfo['continent'] || in_array ($geoInfo['continent'], explode(',', $data)));
        $operator   = $ad == '==';
        return ($expression == $operator);
    } else {
        return ($ad != '==');
    }
}

/*-------------------------------------------------------*/
/* Check if the Referer ACL is valid                     */
/*-------------------------------------------------------*/

function phpAds_aclCheckReferer($data, $ad)
{
    global $_SERVER;   
    if ($data == '') {
        return true;
    }
    $referer = isset($_SERVER['HTTP_REFERER']) ? strtolower($_SERVER['HTTP_REFERER']) : '';
    $expression = strpos($referer, strtolower($data));
    $expression = is_int($expression);
    $operator   = $ad == '==';
    return ($expression == $operator);
}

?>
