<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| Copyright (c) 2000-2003 the phpAdsNew developers                          |
| For contact details, see: http://www.phpadsnew.com/                       |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: zonehistory.plugin.php 2226 2005-03-18 15:56:36Z andrew $
*/

/**
 * @package    MaxPlugins
 * @subpackage Reports
 * @author     Misza Pawlowski <michal@awarez.net>
 * @author     Radek Maciaszek <radek@awarez.net>
 */

require_once MAX_PATH . '/plugins/reports/Reports.php';

class Plugins_Reports_Publisher_Zonehistory extends Plugins_Reports {
    
    // Public info function
    function info()
    {
    	global $strStartDate, $strEndDate, $strZone, $strPluginZone, $strDelimiter;
    	
    	include_once MAX_PATH . '/lib/max/Plugins/PluginTranslation.php';
        Max_PluginTranslation::init($this->module, $this->package);
    	
    	$plugininfo = array (
    		"plugin-name"			=> Max_PluginTranslation::translate('Zone History Report', $this->module, $this->package),
    		"plugin-description"	=> Max_PluginTranslation::translate('Report of publisher and zone distribution for an advertiser.', $this->module, $this->package),
    		'plugin-category'       => 'publisher',
    		'plugin-category-name'	=> Max_PluginTranslation::translate('Publisher Reports', $this->module, $this->package),
    		"plugin-author"			=> "Niels Leenheer",
    		"plugin-export"			=> "csv",
    		"plugin-authorize"		=> phpAds_Admin+phpAds_Agency+phpAds_Affiliate,
    		"plugin-import"			=> array (
    			"campaignid"			=> array (
    				"title"					=> $strZone,
    				"type"					=> "zoneid-dropdown" ),
    			"delimiter"	        => array (
    				"title"					=> $strDelimiter,
    				"type"					=> "edit",
    				"size"					=> 1,
    				"default"				=> "," ),
    			"startdate"				=> array (
                    "title"					=> $strStartDate,
                    "type"					=> "edit",
                    "size"					=> 10,
                    "default"				=> date("Y/m/d",mktime (0,0,0,date("m")-1,1,  date("Y")))
                ),
                "enddate"				=> array (
                    "title"					=> $strEndDate,
                    "type"					=> "edit",
                    "size"					=> 10,
                    "default"				=> date("Y/m/d",mktime (0,0,0,date("m"),1,  date("Y")))
                ),
    		 )
    	);
    	
    	return ($plugininfo);
    }
    
    /*-------------------------------------------------------*/
    /* Private plugin function                               */
    /*-------------------------------------------------------*/
    
    function execute($zoneid, $delimiter=",", $startdate, $enddate)
    {
        $conf = & $GLOBALS['_MAX']['CONF'];
        
    	global $date_format;
    	global $strZone, $strTotal, $strDay, $strViews, $strClicks, $strCTRShort;
    	
    	header ("Content-type: application/csv\nContent-Disposition: inline; filename=zonehistory.csv");
    	
    	$startdate = str_replace('/', '-', $startdate);
    	$enddate   = str_replace('/', '-', $enddate);
    
    	// SQL to generate report data:
        $res_query ="SELECT
            c.campaignname AS campaign,
            DATE_FORMAT(a.day, '%d/%m/%Y') as day,
            SUM(a.impressions) AS adviews,
            SUM(a.clicks) AS adclicks
        FROM
            ".$conf['table']['prefix'].$conf['table']['data_summary_ad_hourly']." AS a,
            ".$conf['table']['prefix'].$conf['table']['banners']." AS b,
            ".$conf['table']['prefix'].$conf['table']['campaigns']." AS c
        WHERE
            a.zone_id = '498'
          AND
            a.ad_id = b.bannerid
          AND
            b.campaignid = c.campaignid
          AND
            day >= '".$startdate."'
          AND
            day <  '".$enddate."'
    	GROUP BY
            day, c.campaignname
        ORDER BY
            campaignname, day";
    
    /*
    $res_query = "
    		SELECT
    			DATE_FORMAT(day, '".$date_format."') as day,
    			SUM(impressions) AS adviews,
    			SUM(clicks) AS adclicks
    		FROM
    			".$conf['tbl_data_summary_ad_hourly']."
    		WHERE
    			zone_id = '".$zoneid."'
    		GROUP BY
    			day
            ORDER BY
                DATE_FORMAT(day, '%Y%m%d')
    	";
    */	
    	$res_banners = phpAds_dbQuery($res_query) or phpAds_sqlDie();
    	
    	echo $strZone.": ".strip_tags(phpAds_getZoneName ($zoneid))."\n\n";
    	echo $strCampaign.$delimiter.$strDay.$delimiter.$strViews.$delimiter.$strClicks.$delimiter.$strCTRShort."\n";
    
    	while ($row_banners = phpAds_dbFetchArray($res_banners))
    	{
    	    /*
    		$stats [$row_banners['day']]['views'] = $row_banners['adviews'];
    		$stats [$row_banners['day']]['clicks'] = $row_banners['adclicks'];
    	}
    	
    	echo $strZone.": ".strip_tags(phpAds_getZoneName ($zoneid))."\n\n";
    	echo $strDay.$delimiter.$strViews.$delimiter.$strClicks.$delimiter.$strCTRShort."\n";
    	
    	$totalclicks = 0;
    	$totalviews = 0;
    	
    	if (isset($stats) && is_array($stats))
    	{
    		for (reset($stats);$key=key($stats);next($stats))
    		{
    			$row = array();
    			
    			//$key = implode('/',array_reverse(split('[-]',$key)));
    			
    			$row[] = $key;
    			$row[] = $stats[$key]['views'];
    			$row[] = $stats[$key]['clicks'];
    			$row[] = phpAds_buildCTR ($stats[$key]['views'], $stats[$key]['clicks']);
    			
    			echo implode ($delimiter, $row)."\n";
    			
    			$totalclicks += $stats[$key]['clicks'];
    			$totalviews += $stats[$key]['views'];
    		}
    	*/
    	
    	echo $row_banners['campaign'].$delimiter.$row_banners['day'].$delimiter.$row_banners['adviews'].$delimiter.$row_banners['adclicks'].$delimiter.phpAds_buildCTR ($row_banners['adviews'], $row_banners['adclicks'])."\n";
    	}
    	
    }
}

?>